// Fetch Examples
// https://jasonwatmore.com/post/2021/09/06/fetch-http-get-request-examples

class APIManager {
  #baseUrl;

  // The default time to wait before timing out a fetch call.
  static get DEFAULT_FETCH_TIMEOUT() { return 2000; }

  // The default time to wait before trying to subscribe to the streaming endpoint again.
  static get DEFAULT_STREAMING_RETRY_TIME() { return 1000; }

  // Used to abort the fetch of the streaming endpoints.
  #streamingAbortController;

  // Keys are `StreamingEndpoint` paths and values are callbacks.
  #streamingCallbacks;

  // Used to retry subscribing to streaming endpoints after a certain time.
  #streamingRetryTimeout;

  #isProcessingUpdateStream;

  constructor(baseurl) {
    this.#baseUrl = baseurl;
    this.#streamingAbortController = null;
    this.#streamingRetryTimeout = null;
    this.#streamingCallbacks = {};
  }

  set baseUrl(baseUrl) {
    this.#baseUrl = baseUrl;

    if (baseUrl !== '') {
      this.subscribeToStreamingEndpoints();
    } else if (this.#streamingAbortController) {
      this.#streamingAbortController.abort();
    }
  }

  get baseUrl() {
    return this.#baseUrl;
  }

  // *** Update Streaming ***

  /**
   *
   * @param {StreamingEndpoint} endpoint - The endpoint to register a callback for.
   * @param {*} callback - The function to be called when new data is received for a given `StreamingEndpoint`.
   */
  registerCallbackForStreamingEndpoint(endpoint, callback) {
    // Currently this supports only one callback per endpoint. Could be expanded if needed in the future.
    this.#streamingCallbacks[endpoint.path] = callback;
  }

  /**
   * Retries streaming endpoint subscription.
   * If a retry is already queued via the `#streamingRetyTimeout`, this method does nothing.
   * @param {number} time - The amount of time in ms to wait before retrying subscribing to streaming endpoints.
   */
  retrySubscribingAfterTime(time = APIManager.DEFAULT_STREAMING_RETRY_TIME) {
    if (this.#streamingRetryTimeout || this.#baseUrl === '') {
      return;
    }

    this.#streamingRetryTimeout = setTimeout(() => { this.subscribeToStreamingEndpoints(); }, time);
  }

  /**
   * Initiates fetch that subscribes to the streaming status update endpoint.
   */
  subscribeToStreamingEndpoints() {
    this.#isProcessingUpdateStream = false;
    if (this.#streamingAbortController) {
      // Close the previous connection so we only have one open at a time.
      this.#streamingAbortController.abort();
    }
    if (this.#streamingRetryTimeout) {
      // Cancel any outstanding attempt to retry.
      clearTimeout(this.#streamingRetryTimeout);
      this.#streamingRetryTimeout = null;
    }

    const statusUrl = `${this.baseUrl}/v1/status/updates`;
    const endpoints = Object.keys(this.#streamingCallbacks);
    this.#streamingAbortController = new AbortController();

    fetch(statusUrl, {
      method: 'POST',
      headers: new Headers({ 'content-type': 'application/json' }),
      body: JSON.stringify(endpoints),
      signal: this.#streamingAbortController.signal,
    })
      .then((response) => {
        if (!response.ok) {
          return response.text().then((text) => {
            throw new Error(`HTTP Error! Status: ${response.status} Body: ${text}`);
          });
        }
        return this.processUpdateStream(response);
      })
      .catch((error) => {
        console.warn(statusUrl, error);
        const isNetworkError = error.name === 'TypeError' && error.message === 'Failed to fetch';
        if (isNetworkError || !this.#isProcessingUpdateStream) {
          this.retrySubscribingAfterTime();
        }
      });
  }

  /**
   * Processes the update stream response by recursively reading chunks of data.
   * @param {*} response - The response from fetching the status update endpoint.
   */
  processUpdateStream(response) {
    this.#isProcessingUpdateStream = true;
    const self = this;

    const reader = response.body.getReader();
    const decoder = new TextDecoder('utf-8');
    let buffer = '';
    const delimiter = '\r\n\r\n';

    function readChunk() {
      reader.read()
        // eslint-disable-next-line no-use-before-define
        .then(processChunk)
        .catch((error) => {
          if (error.name !== 'AbortError') {
            self.retrySubscribingAfterTime();
            console.warn(`Error reading chunk: ${error}`);
          }
        });
    }

    function processChunk(result) {
      buffer += decoder.decode(result.value || new Uint8Array(), { stream: !result.done });

      while (true) {
        const index = buffer.indexOf(delimiter);
        if (index < 0) {
          break;
        }
        const chunk = buffer.substring(0, index);
        buffer = buffer.substring(index + delimiter.length);

        try {
          const obj = JSON.parse(chunk);
          const callback = self.#streamingCallbacks[obj.url];
          if (callback) {
            callback(obj.data);
          }
        } catch (error) {
          console.warn(`Error processing chunk: ${error}, ${chunk}`);
          self.retrySubscribingAfterTime();
        }
      }

      if (!result.done) {
        readChunk();
      } else {
        console.error('Status update connection closed unexpectedly.');
        self.retrySubscribingAfterTime();
      }
    }

    readChunk();
  }

  // *** Audio Playlist Action ***

  /**
     * Gets all audio playlists
     * @throws
     */
  async getAudioPlaylists() {
    const url = `${this.baseUrl}/v1/audio/playlists`;
    const response = await this.fetchWithTimeout(url).then(APIManager.handleErrors);
    return response.json();
  }

  /**
    * Gets all items in the audio playlist with the specified ID.
    * @param {string} playlistID - The ID of the playlist to get the items of.
    */
  async getAudioItemsInPlaylist(playlistID) {
    let allItems = [];
    let moreItemsAvailable = true;

    /* eslint-disable no-await-in-loop */
    while (moreItemsAvailable) {
      const url = `${this.baseUrl}/v1/audio/playlist/${playlistID}?start=${allItems.length}`;
      const response = await this.fetchWithTimeout(url).then(APIManager.handleErrors);
      const playlist = await response.json();
      allItems = allItems.concat(playlist.items);

      // If the number of items returned is less than 100, there are no more items to fetch
      if (playlist.items.length < 100) {
        moreItemsAvailable = false;
      }
    }
    /* eslint-enable no-await-in-loop */
    return allItems;
  }

  /**
     * Triggers the audio playlist with the specified ID.
     * @param {string} playlistID - The ID of the playlist to trigger.
     * @throws
     */
  async triggerAudio(playlistID, itemID = null) {
    let triggerURL;

    if (itemID != null) {
      triggerURL = `${this.baseUrl}/v1/audio/playlist/${playlistID}/${itemID}/trigger`;
    } else {
      triggerURL = `${this.baseUrl}/v1/audio/playlist/${playlistID}/trigger`;
    }

    return this.fetchWithTimeout(triggerURL).then(APIManager.handleErrors);
  }

  // *** Stage Action ***

  /**
    * Gets an array of all configured stage screens.
    * @throws
    */
  async getStageScreens() {
    const url = `${this.baseUrl}/v1/stage/screens`;
    const response = await this.fetchWithTimeout(url).then(APIManager.handleErrors);
    return response.json();
  }

  /**
  * Gets an array of all configured stage layouts.
  * @throws
  */
  async getStageLayouts() {
    const url = `${this.baseUrl}/v1/stage/layouts`;
    const response = await this.fetchWithTimeout(url).then(APIManager.handleErrors);
    return response.json();
  }

  /**
  * Sets the specified layouts on the corresponding stage screens.
  * @param {Array.<Object>} layouts - An array of objects each containing a pair of objects that identify
  * the screen and the layout to set on that screen.
  * @throws
  */
  async setStageLayouts(layouts) {
    const url = `${this.baseUrl}/v1/stage/layout_map`;
    return this.fetchWithTimeout(url, {
      method: 'PUT',
      headers: new Headers({ 'content-type': 'application/json' }),
      body: JSON.stringify(layouts),
    }).then(APIManager.handleErrors);
  }

  /**
     * Shows the specified message on any stage displays with text links to the stage message.
     * @param {string} message
     * @throws
     */
  async showStageMessage(message) {
    const url = `${this.baseUrl}/v1/stage/message`;
    return this.fetchWithTimeout(url, {
      method: 'PUT',
      headers: new Headers({ 'content-type': 'application/json' }),
      body: `"${message}"`,
    }).then(APIManager.handleErrors);
  }

  /**
     * Hides any currently displaying stage message from all stage screens
     * @throws
     */
  async hideStageMessage() {
    const url = `${this.baseUrl}/v1/stage/message`;
    return this.fetchWithTimeout(url, { method: 'DELETE' }).then(APIManager.handleErrors);
  }

  // *** Slide Action ***

  /**
     * Triggers the next slide in the current presentation.
     * @throws
     */
  async triggerNextSlide() {
    const url = `${this.baseUrl}/v1/trigger/next`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Triggers the previous slide in the current presentation.
     * @throws
     */
  async triggerPreviousSlide() {
    const url = `${this.baseUrl}/v1/trigger/previous`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  // *** Find My Mouse Action ***

  /**
     * Brings the mouse cursor to the center of screen.
     * @throws
     */
  async findMyMouse() {
    const url = `${this.baseUrl}/v1/find_my_mouse`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  // *** Capture Action ***

  /**
     * Starts capture recording.
     * @throws
     */
  async startCapture() {
    const url = `${this.baseUrl}/v1/capture/start`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Stops capture recording.
     * @throws
     */
  async stopCapture() {
    const url = `${this.baseUrl}/v1/capture/stop`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  // *** Timer Action ***

  /**
     * Gets an array of all configured timers.
     * This endpoint is apparently supposed to be a chunked response, and so will break when that is changed.
     * @throws
     */
  async getTimers() {
    const url = `${this.baseUrl}/v1/timers`;
    const response = await this.fetchWithTimeout(url).then(APIManager.handleErrors);
    return response.json();
  }

  /**
     * Starts the specified timer.
     * @param {string} timerId - The ID of the timer to start.
     * @param {*} configuration - Optional configuration to set when starting the timer.
     * @throws
     */
  async startTimer(timerId, configuration) {
    const url = `${this.baseUrl}/v1/timer/${timerId}/start`;

    if (configuration) {
      return this.fetchWithTimeout(url, {
        method: 'PUT',
        headers: new Headers({ 'content-type': 'application/json' }),
        body: JSON.stringify(configuration),
      }).then(APIManager.handleErrors);
    }

    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Stops the specified timer.
     * @param {string} timerId - The ID of the timer to stop.
     * @param {*} configuration - Optional configuration to set when stopping the timer.
     * @throws
     */
  async stopTimer(timerId, configuration) {
    const url = `${this.baseUrl}/v1/timer/${timerId}/stop`;

    if (configuration) {
      return this.fetchWithTimeout(url, {
        method: 'PUT',
        headers: new Headers({ 'content-type': 'application/json' }),
        body: JSON.stringify(configuration),
      }).then(APIManager.handleErrors);
    }

    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Resets the specified timer.
     * @param {string} timerId - The ID of the timer to reset.
     * @param {*} configuration - Optional configuration to set when resetting the timer.
     * @throws
     */
  async resetTimer(timerId, configuration) {
    const url = `${this.baseUrl}/v1/timer/${timerId}/reset`;

    if (configuration) {
      return this.fetchWithTimeout(url, {
        method: 'PUT',
        headers: new Headers({ 'content-type': 'application/json' }),
        body: JSON.stringify(configuration),
      }).then(APIManager.handleErrors);
    }

    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Increments the specified timer.
     * @param {string} timerId - The ID of the timer to reset.
     * @param {number} amount - The amount to increment the timer by.
     * @throws
     */
  async incrementTimer(timerId, amount) {
    const url = `${this.baseUrl}/v1/timer/${timerId}/increment/${amount}`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  // *** Look Action ***

  /**
     * Gets an array of all configured audience looks.
     * @throws
     */
  async getLooks() {
    const url = `${this.baseUrl}/v1/looks`;
    const response = await this.fetchWithTimeout(url).then(APIManager.handleErrors);
    return response.json();
  }

  /**
     * Triggers the audience look with the specified ID.
     * @param {string} lookID - The ID of the look to trigger.
     * @throws
     */
  async triggerLook(lookID) {
    const url = `${this.baseUrl}/v1/look/${lookID}/trigger`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  // *** Macro Action ***

  /**
     * Gets an array of all configured macros.
     * @throws
     */
  async getMacros() {
    const url = `${this.baseUrl}/v1/macros`;
    const response = await this.fetchWithTimeout(url).then(APIManager.handleErrors);
    return response.json();
  }

  /**
     * Triggers the macro with the specified ID.
     * @param {string} macroID - The ID of the macro to trigger.
     * @throws
     */
  async triggerMacro(macroID) {
    const url = `${this.baseUrl}/v1/macro/${macroID}/trigger`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  // *** Transport Action ***

  /**
     * Starts media playback on the specified layer.
     * @param {string} layer - The layer to start playback on.
     * Valid layers : `presentation`, `announcment`, `audio`
     * @throws
     */
  async transportPlay(layer) {
    const url = `${this.baseUrl}/v1/transport/${layer}/play`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Pauses media playback on the specified layer.
     * @param {string} layer - The layer to pause playback on.
     * Valid layers : `presentation`, `announcment`, `audio`
     * @throws
     */
  async transportPause(layer) {
    const url = `${this.baseUrl}/v1/transport/${layer}/pause`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Skips backward on the specified layer by the given amount of time.
     * @param {string} layer - The layer to skip backward on.
     * Valid layers : `presentation`, `announcment`, `audio`
     * @param {number} time - The amount of time in seconds to skip backward.
     * @throws
     */
  async transportSkipBackward(layer, time) {
    const url = `${this.baseUrl}/v1/transport/${layer}/skip_backward/${time}`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Skips forward on the specified layer by the given amount of time.
     * @param {string} layer - The layer to skip forward on.
     * Valid layers : `presentation`, `announcment`, `audio`
     * @param {number} time - The amount of time in seconds to skip forward.
     * @throws
     */
  async transportSkipForward(layer, time) {
    const url = `${this.baseUrl}/v1/transport/${layer}/skip_forward/${time}`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Moves to the end on the specified layer minus the given amount of time.
     * @param {string} layer - The layer to go the end of.
     * Valid layers : `presentation`, `announcment`, `audio`
     * @param {number} time - The time offset in seconds from the end of the media.
     * @throws
     */
  async transportGoToEnd(layer, time) {
    const url = `${this.baseUrl}/v1/transport/${layer}/go_to_end?time=${time}`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  // *** Prop Action ***

  /**
     * Gets an array of all configured props.
     * @throws
     */
  async getProps() {
    const url = `${this.baseUrl}/v1/props`;
    const response = await this.fetchWithTimeout(url).then(APIManager.handleErrors);
    return response.json();
  }

  /**
     * Triggers the specified prop.
     * @param {string} propID - The ID of the prop to trigger.
     * @throws
     */
  async triggerProp(propID) {
    const url = `${this.baseUrl}/v1/prop/${propID}/trigger`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Clears the specified prop.
     * @param {string} propID - The ID of the prop to clear.
     * @throws
     */
  async clearProp(propId) {
    const url = `${this.baseUrl}/v1/prop/${propId}/clear`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  // *** Group Action ***

  /**
     * @throws
     * @returns {Array.<Object>} - An array of group objects, each with an ID and color.
     */
  async getGroups() {
    const url = `${this.baseUrl}/v1/groups`;
    const response = await this.fetchWithTimeout(url).then(APIManager.handleErrors);
    return response.json();
  }

  /**
     * @param {string} groupID - The ID of the arrangment group to trigger.
     * @throws
     */
  async triggerGroup(groupID) {
    const url = `${this.baseUrl}/v1/group/${groupID}`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  // *** Clear Action ***

  /**
     * Gets an array of all configured clear groups.
     * @throws
     */
  async getClearGroups() {
    const url = `${this.baseUrl}/v1/clear/groups`;
    const response = await this.fetchWithTimeout(url).then(APIManager.handleErrors);
    return response.json();
  }

  /**
     * Triggers the clear group with the specified name.
     * @param {string} clearGroupName - The name of the clear group to trigger.
     * @throws
     */
  async triggerClearGroup(clearGroupName) {
    const url = `${this.baseUrl}/v1/clear/group/${clearGroupName}/trigger`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Clears the specified layer on all outputs.
     * @param {string} layerToClear - The name of the layer to clear.
     * Valid layers : `audio`, `props`, `messages`, `announcements`, `slide`, `media`, `video_input`
     * @throws
     */
  async triggerClearLayer(layerToClear) {
    const url = `${this.baseUrl}/v1/clear/layer/${layerToClear}`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  // *** Timeline Action ***

  /**
     * Starts playback of the timeline for the focused presentation.
     * @throws
     */
  async timelinePlay() {
    const url = `${this.baseUrl}/v1/presentation/focused/timeline/play`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Pauses playback of the timeline for the focused presentation.
     * @throws
     */
  async timelinePause() {
    const url = `${this.baseUrl}/v1/presentation/focused/timeline/pause`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Goes to the beginning of the timeline for the focused presentation.
     * @throws
     */
  async timelineRewind() {
    const url = `${this.baseUrl}/v1/presentation/focused/timeline/rewind`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  // *** Video Input Action ***

  /**
     * Gets an array of all configured video inputs.
     * @throws
     */
  async getVideoInputs() {
    const url = `${this.baseUrl}/v1/video_inputs`;
    const response = await this.fetchWithTimeout(url).then(APIManager.handleErrors);
    return response.json();
  }

  /**
     * Triggers the video input with the specified ID.
     * @param {string} inputID - The ID of the video input to trigger.
     */
  async triggerVideoInput(inputID) {
    const url = `${this.baseUrl}/v1/trigger/video_input/${inputID}`;
    return this.fetchWithTimeout(url).then(APIManager.handleErrors);
  }

  /**
     * Checks a response status code for a server error (status outside of 200-299) which if found will result in throwing an Error with the status text.
     * @param {Response} response - The response to check for server errors.
     * @returns {Response} The response passed in to this method, unmodified.
     */
  static handleErrors(response) {
    if (!response.ok) {
      throw Error(response.statusText);
    }
    return response;
  }

  async fetchWithTimeout(resource, options = {}) {
    if (this.#baseUrl === '') {
      throw new Error('No connection set up');
    }
    const { timeout = APIManager.DEFAULT_FETCH_TIMEOUT } = options;
    const abortController = new AbortController();
    const id = setTimeout(() => abortController.abort(), timeout);
    const response = await fetch(resource, {
      ...options,
      signal: abortController.signal,
    });
    clearTimeout(id);
    return response;
  }
}

if (typeof module !== 'undefined') {
  module.exports = APIManager;
}
