/* global $SD, Utils, KEY_ICONS */

class ActionBase {
  apiManager;

  // A boolean used to indcate if this action should poll for data while the property inspector is visible.
  // If true, `pollForData` will be called repeatedly while the property inspector is displayed.
  shouldPollForData;

  // Private timer used for polling for data from API.
  #pollingTimeout;

  // Public getter for testing purposes
  get pollingTimeout() { return this.#pollingTimeout; }

  // Private boolean used to indicate if the action is currently polling for data.
  #isPolling;

  // Public getter for testing purposes
  get isPolling() { return this.#isPolling; }

  // A dictionary of contexts of the action instances that are currently visible on the Stream Deck hardware.
  visibleInstances;

  // Time interval to wait between calls for data when polling.
  static get POLL_TIME() { return 1000; }

  constructor(apiManager) {
    this.apiManager = apiManager;
    this.shouldPollForData = false;
    this.#isPolling = false;
    this.#pollingTimeout = null;
    this.visibleInstances = {};
  }

  // *** Events ***

  // eslint-disable-next-line class-methods-use-this
  onDidReceiveSettings(json) {
    ActionBase.updateKeyIcon(json.context, json.payload.settings, json.action);
  }

  /**
     * The 'willAppear' event is the first event a key will receive, right before it gets
     * shown on your Stream Deck and/or in Stream Deck software.
     * This event is a good place to setup your plugin and look at current settings (if any),
     * which are embedded in the events payload.
     */
  onWillAppear(json) {
    this.visibleInstances[json.context] = json.context;
    const { settings } = json.payload;
    const didSetDefaults = this.setDefaultsInSettings(settings);

    if (didSetDefaults) {
      ActionBase.saveSettings(json.context, settings);
    }

    $SD.api.getGlobalSettings($SD.uuid);
    ActionBase.updateKeyIcon(json.context, settings, json.action);
  }

  onWillDisappear(json) {
    delete this.visibleInstances[json.context];
  }

  // eslint-disable-next-line class-methods-use-this, no-unused-vars, no-empty-function
  onKeyDown(json) { }

  async onKeyUp(json) {
    try {
      await this.performAction(json.payload.settings);
    } catch (error) {
      console.warn(`Error performing action: ${error}`);
      ActionBase.showAlert(json.context);
    }
  }

  onPropertyInspectorDidAppear(json) {
    this.startPolling(json.context, json.action);
  }

  // eslint-disable-next-line no-unused-vars, no-empty-function
  onPropertyInspectorDidDisappear(json) {
    this.stopPolling();
  }

  /**
     * This is a message sent directly from the Property Inspector
     * (e.g. some value, which is not saved to settings)
     * You can send this event from Property Inspector (see there for an example)
     */
  // eslint-disable-next-line class-methods-use-this, no-unused-vars, no-empty-function
  onSendToPlugin(json) { }

  /**
     * @param {Object} settings - The settings object to update with defaults for an action.
     * @returns A boolean indicating whether or not settings were updated.
     * When subclasssing be sure to return `super.setDefaultsInSettings(settings) || didUpdateSettings`
     */
  // eslint-disable-next-line class-methods-use-this
  setDefaultsInSettings(settings) {
    let didUpdateSettings = false;

    if (typeof settings['icon-background-color'] === 'undefined') {
      settings['icon-background-color'] = null;
      didUpdateSettings = true;
    }

    if (typeof settings['icon-tint'] === 'undefined') {
      settings['icon-tint'] = 'lightIcon';
      didUpdateSettings = true;
    }

    if (typeof settings.actionColor === 'undefined') {
      settings.actionColor = null;
      didUpdateSettings = true;
    }

    return didUpdateSettings;
  }

  /**
     * This method should be subclassed by each action in order to perform it's function on the user input.
     * The subclass does not need to call super as the method is not implemented in ActionBase.
     * @param {Object} settings - The settings to reference for information required to perform the action.
     * @throws - An error if unable to perform the action successfully.
     */
  // eslint-disable-next-line class-methods-use-this, no-unused-vars, no-empty-function
  async performAction(settings) { }

  /**
     * Sets the key icon to the appropriate image with the selected background color.
     *
     * @param {string} context
     * @param {Object} settings
     * @param {string} action
     */
  static updateKeyIcon(context, settings, action) {
    const iconState = settings['icon-state'];
    const iconTint = settings['icon-tint'];
    const backgroundColor = settings['icon-background-color'];
    // A color that comes from the API that we will use if the user didn't explicitly choose a bg color.
    const { actionColor } = settings;
    const { stateColor } = settings;

    let imagePath;
    if (typeof iconState === 'undefined') {
      // Some actions don't utitilize icon state
      imagePath = KEY_ICONS[`${action}`][`${iconTint}`];
    } else {
      let pathsForState = KEY_ICONS[`${action}`][`${iconState}`];
      if (typeof pathsForState === 'undefined') {
        // Icon state can be set from data coming from the API, which could be incorrect.
        // In that case, we will just select the first possible icon state.
        console.error(`Invalid icon state: ${iconState}`);
        [pathsForState] = Object.values(KEY_ICONS[`${action}`]);
      }
      imagePath = pathsForState[`${iconTint}`];
    }

    // eslint-disable-next-line max-len
    Utils.loadImage(imagePath, (image) => ActionBase.setImage(context, image), true, stateColor || backgroundColor || actionColor);
  }

  // *** Polling ***

  /**
     * Begins polling for data from the API. Calls `pollForData` which is up to subclasses to implement.
     *
     * @param {string} context
     * @param {string} action
     * @returns
     */
  startPolling(context, action) {
    if (!this.shouldPollForData) { return; }

    this.#isPolling = true;
    this.pollForData(context, action);
  }

  /**
     * Stops polling for data from the API.
     */
  stopPolling() {
    clearTimeout(this.#pollingTimeout);
    this.#pollingTimeout = null;
    this.#isPolling = false;
  }

  /**
     * This method should be subclassed by any action that sets `shouldPollForData` to true.
     * It is up to the subclass to fetch data from the API, handle any errors, and send the results to the property inspector.
     * At the end of the method, the subclass should call super.
     *
     * @param {string} context
     * @param {string} action
     */
  // eslint-disable-next-line class-methods-use-this, no-unused-vars, no-empty-function
  async pollForData(context, action) {
    if (!this.#isPolling) {
      return;
    }
    this.#pollingTimeout = setTimeout(() => { this.pollForData(context, action); }, ActionBase.POLL_TIME);
  }

  // *** System methods ***

  /**
   * Gets the settings from Stream Deck for each of the visible instances of this action.
   */
  getSettings() {
    Object.values(this.visibleInstances).forEach((context) => {
      $SD.api.getSettings(context);
    });
  }

  /**
   *
   * Sets and saves a key value pair in settings only if necessary.
   *
   * @param {string} key The settings key to set the value for.
   * @param {*} value The value to store for `key`.
   * @param {Object} settings The settings object to store the key value pair in.
   * @param {*} context The context of the action instance.
   */
  static setSetting(key, value, settings, context) {
    if (settings[key] === value) {
      return;
    }

    settings[key] = value;
    ActionBase.saveSettings(context, settings);
  }

  // Used to persist the settings with the Stream Deck hardware for a specific action's context
  static saveSettings(context, settings) {
    $SD.api.setSettings(context, settings);
  }

  // Sets the actions title to the passed in title
  static setTitle(context, title) {
    $SD.api.setTitle(context, title);
  }

  // Clears the actions title
  static clearTitle(context) {
    $SD.api.setTitle(context, '');
  }

  // Sets the actions key action icon
  static setImage(context, image) {
    $SD.api.setImage(context, image);
  }

  // Shows a warning logo over the key image
  static showAlert(context) {
    $SD.api.showAlert(context);
  }

  // Sends an object to the property inspector
  static sendToPropertyInspector(context, payload, action) {
    $SD.api.sendToPropertyInspector(context, payload, action);
  }
}

if (typeof module !== 'undefined') {
  module.exports = ActionBase;
}
