/* global ActionBase, StreamingEndpoint */

class CaptureAction extends ActionBase {
  #captureStatus;

  get captureStatus() { return this.#captureStatus; }

  static get ACTIVE_COLOR() { return '#25A41C'; }

  static get CAUTION_COLOR() { return '#A4851C'; }

  static get ERROR_COLOR() { return '#D01F1F'; }

  constructor(apiManager) {
    super(apiManager);
    this.shouldPollForData = true;
    this.#captureStatus = {};

    apiManager.registerCallbackForStreamingEndpoint(StreamingEndpoint.Capture, (status) => {
      this.#captureStatus = status;
      this.getSettings(); // Get settings so that key appearance can be updated
    });
  }

  async onDidReceiveSettings(json) {
    this.#updateTitle(json.payload.settings, json.context);
    this.#updateIconState(json.payload.settings, json.context);
    this.#updateStateColor(json.payload.settings);

    super.onDidReceiveSettings(json);
  }

  setDefaultsInSettings(settings) {
    let didUpdateSettings = false;

    if (typeof settings.modeSelector === 'undefined') {
      settings.modeSelector = 'StartCapture';
      didUpdateSettings = true;
    }

    if (typeof settings['icon-state'] === 'undefined') {
      settings['icon-state'] = 'start';
      didUpdateSettings = true;
    }

    return super.setDefaultsInSettings(settings) || didUpdateSettings;
  }

  async performAction(settings) {
    const modeSelection = settings.modeSelector;
    if (typeof modeSelection === 'undefined') {
      throw new Error('modeSelector not defined in settings');
    }

    switch (modeSelection) {
      case ('StartCapture'):
        await this.apiManager.startCapture();
        break;
      case ('StopCapture'):
        await this.apiManager.stopCapture();
        break;
      case ('ToggleCapture'):
        if (this.#isCapturing()) {
          await this.apiManager.stopCapture();
        } else {
          await this.apiManager.startCapture();
        }
        break;
      default:
        throw new Error(`Unhandled action selection: ${modeSelection}`);
    }
  }

  // *** Private Methods ***

  /**
   * Sets the title to the capture time when in the toggle mode.
   * @param {Object} settings - The current settings for this action instance.
   * @param {*} context - The context of this action instance.
   */
  #updateTitle = (settings, context) => {
    if (settings.modeSelector === 'ToggleCapture') {
      if (this.#isCapturing()) {
        let captureTime = this.#captureStatus.capture_time;
        if (typeof captureTime === 'undefined') {
          console.warn('No capture status to get time from');
          return;
        }
        captureTime = captureTime.replace(/^(0:)/, ''); // Strip leading zero
        captureTime = captureTime.replace(/^(00:)/, '0:'); // Replace double leading zero with single zero
        ActionBase.setTitle(context, captureTime);
      } else {
        ActionBase.clearTitle(context);
      }
    } else {
      ActionBase.clearTitle(context);
    }
  };

  /**
   * Sets the appropriate `icon-state` in settings and also saves the settings if necessary.
   * @param {Object} settings - The current settings for this action instance.
   * @param {*} context - The context of this action instance.
   */
  #updateIconState = (settings, context) => {
    switch (settings.modeSelector) {
      case 'StartCapture':
        ActionBase.setSetting('icon-state', 'start', settings, context);
        break;
      case 'StopCapture':
        ActionBase.setSetting('icon-state', 'stop', settings, context);
        break;
      case 'ToggleCapture':
        try {
          if (this.#isCapturing()) {
            ActionBase.setSetting('icon-state', 'stop', settings, context);
          } else {
            ActionBase.setSetting('icon-state', 'start', settings, context);
          }
        } catch (error) {
          console.warn(error);
        }
        break;
      default:
        console.error(`Unhandled case: ${settings.modeSelector}`);
        break;
    }
  };

  /**
   * Sets the action color in the settings.
   * @param {*} settings - The current settings for this action instance.
   */
  #updateStateColor = (settings) => {
    if (settings.modeSelector !== 'ToggleCapture') {
      settings.stateColor = null;
      return;
    }

    switch (this.#captureStatus.status) {
      case 'active':
        settings.stateColor = CaptureAction.ACTIVE_COLOR;
        break;
      case 'inactive':
        settings.stateColor = null;
        break;
      case 'caution':
        settings.stateColor = CaptureAction.CAUTION_COLOR;
        break;
      case 'error':
        settings.stateColor = CaptureAction.ERROR_COLOR;
        break;
      default:
        break;
    }
  };

  /**
   * @throws An error if there is no capture status.
   * @returns True if capture is ongoing, false otherwise.
   */
  #isCapturing = () => {
    const { status } = this.#captureStatus;

    if (typeof status === 'undefined') {
      throw new Error('No capture status');
    }

    return (status === 'active' || status === 'caution');
  };
}

if (typeof module !== 'undefined') {
  module.exports = CaptureAction;
}
