/* global ActionBase, StreamingEndpoint */

class ClearAction extends ActionBase {
  // The status of layers as returned from `StreamingEndpoint.Layers`
  #layerStatus;

  // Public readonly access of layer status, needed for testing
  get layerStatus() { return this.#layerStatus; }

  static get ACTIVE_CONTENT_COLOR() { return '#8D2A2A'; }

  constructor(apiManager) {
    super(apiManager);
    this.shouldPollForData = true;
    this.#layerStatus = {};

    apiManager.registerCallbackForStreamingEndpoint(StreamingEndpoint.Layers, (layerStatus) => {
      this.#layerStatus = layerStatus;
      this.getSettings(); // Get settings so that action color can be updated
    });
  }

  onDidReceiveSettings(json) {
    const { settings } = json.payload;
    // `actionColor` is not persisted in settings as it is transient in nature.
    settings.stateColor = this.#actionWouldClearContent(settings) ? ClearAction.ACTIVE_CONTENT_COLOR : null;
    super.onDidReceiveSettings(json);
  }

  setDefaultsInSettings(settings) {
    let didUpdateSettings = false;

    if (typeof settings['icon-state'] === 'undefined') {
      settings['icon-state'] = 'X';
      didUpdateSettings = true;
    }

    return super.setDefaultsInSettings(settings) || didUpdateSettings;
  }

  async pollForData(context, action) {
    try {
      const clearGroups = await this.apiManager.getClearGroups();
      ActionBase.sendToPropertyInspector(context, { availableClearGroups: clearGroups }, action);
    } catch (error) {
      console.warn(error);
      ActionBase.sendToPropertyInspector(context, { availableClearGroups: null }, action);
    }

    super.pollForData(context, action);
  }

  async performAction(settings) {
    const clearValue = settings.modeSelector;

    if (typeof clearValue === 'undefined') {
      throw new Error('clearValue not defined in settings');
    }

    switch (clearValue) {
      case 'Audio-Layer':
        await this.apiManager.triggerClearLayer('audio');
        break;
      case 'Message-Layer':
        await this.apiManager.triggerClearLayer('messages');
        break;
      case 'Prop-Layer':
        await this.apiManager.triggerClearLayer('props');
        break;
      case 'Announcement-Layer':
        await this.apiManager.triggerClearLayer('announcements');
        break;
      case 'Slide-Layer':
        await this.apiManager.triggerClearLayer('slide');
        break;
      case 'Media-Layer':
        await this.apiManager.triggerClearLayer('media');
        break;
      case 'Video-Input-Layer':
        await this.apiManager.triggerClearLayer('video_input');
        break;
      default:
        // If we hit the default then we are attempting to trigger a clear group
        await this.apiManager.triggerClearGroup(clearValue);
        break;
    }
  }

  // *** Private Methods ***

  /**
   * @param {Object} settings - The current settings for this action instance.
   * @returns True if performing the action would clear content, false otherwise.
   */
  #actionWouldClearContent = (settings) => {
    switch (settings.modeSelector) {
      case 'Audio-Layer':
        return this.#layerStatus.audio;
      case 'Message-Layer':
        return this.#layerStatus.messages;
      case 'Prop-Layer':
        return this.#layerStatus.props;
      case 'Announcement-Layer':
        return this.#layerStatus.announcements;
      case 'Slide-Layer':
        return this.#layerStatus.slide;
      case 'Media-Layer':
        return this.#layerStatus.media;
      case 'Video-Input-Layer':
        return this.#layerStatus.video_input;
      default:
        break;
    }

    const { selectedClearGroup } = settings;

    if (typeof selectedClearGroup === 'undefined' || selectedClearGroup === null) {
      console.error('There should be a selected clear group if a layer is not selected');
      return false;
    }

    const activeLayers = Object.keys(this.#layerStatus).filter((layer) => this.#layerStatus[layer]);
    const clearGroupLayers = selectedClearGroup.layers.map((layer) => ClearAction.#layerForClearLayer(layer));

    const layerMatch = activeLayers.find((layer) => clearGroupLayers.includes(layer));
    return typeof layerMatch !== 'undefined';
  };

  static #layerForClearLayer = (clearLayer) => {
    switch (clearLayer) {
      case 'music':
      case 'audio-effects':
        return 'audio';
      case 'presentation':
        return 'slide';
      case 'presentation_media':
        return 'media';
      default:
        return clearLayer;
    }
  };
}

if (typeof module !== 'undefined') {
  module.exports = ClearAction;
}
