/* global ActionBase, StreamingEndpoint */

class PropAction extends ActionBase {
  // An array of the current props from the API
  #propStatus;

  // Public readonly access of prop status, needed for testing
  get propStatus() { return this.#propStatus; }

  constructor(apiManager) {
    super(apiManager);
    this.shouldPollForData = true;
    this.#propStatus = [];

    apiManager.registerCallbackForStreamingEndpoint(StreamingEndpoint.Props, (propStatus) => {
      this.#propStatus = propStatus;
      this.getSettings(); // Get settings so that icon can be updated
    });
  }

  onDidReceiveSettings(json) {
    this.#updateIconState(json.payload.settings, json.context);
    super.onDidReceiveSettings(json);
  }

  setDefaultsInSettings(settings) {
    let didUpdateSettings = false;

    if (typeof settings['icon-state'] === 'undefined') {
      settings['icon-state'] = 'trigger';
      didUpdateSettings = true;
    }

    if (typeof settings.modeSelector === 'undefined') {
      settings.modeSelector = 'TriggerProp';
    }

    return super.setDefaultsInSettings(settings) || didUpdateSettings;
  }

  async pollForData(context, action) {
    try {
      const props = await this.apiManager.getProps();
      ActionBase.sendToPropertyInspector(context, { availableProps: props }, action);
    } catch (error) {
      console.warn(error);
      ActionBase.sendToPropertyInspector(context, { availableProps: null }, action);
    }

    super.pollForData(context, action);
  }

  async performAction(settings) {
    const propID = settings.propSelector;

    if (typeof propID === 'undefined') {
      throw new Error('propSelector not defined in settings');
    }

    const modeSelection = settings.modeSelector;

    if (typeof modeSelection === 'undefined') {
      throw new Error('modeSelector not defined in settings');
    }

    switch (modeSelection) {
      case ('TriggerProp'):
        await this.apiManager.triggerProp(propID);
        break;
      case ('ClearProp'):
        await this.apiManager.clearProp(propID);
        break;
      case ('ToggleProp'):
        if (this.#propIsActive(propID)) {
          await this.apiManager.clearProp(propID);
        } else {
          await this.apiManager.triggerProp(propID);
        }
        break;
      default:
        throw new Error(`Unhandled action selection: ${modeSelection}`);
    }
  }

  // *** Private Methods ***

  /**
   * Sets the appropriate `icon-state` in settings and also saves the settings if necessary.
   * @param {Object} settings - The current settings for this action instance.
   * @param {*} context - The context of this action instance.
   */
  #updateIconState = (settings, context) => {
    switch (settings.modeSelector) {
      case 'TriggerProp':
        ActionBase.setSetting('icon-state', 'trigger', settings, context);
        break;
      case 'ClearProp':
        ActionBase.setSetting('icon-state', 'clear', settings, context);
        break;
      case 'ToggleProp':
        if (typeof settings.propSelector === 'undefined') {
          console.warn('propSelector not defined in settings');
          break;
        }

        try {
          if (this.#propIsActive(settings.propSelector)) {
            ActionBase.setSetting('icon-state', 'clear', settings, context);
          } else {
            ActionBase.setSetting('icon-state', 'trigger', settings, context);
          }
        } catch (error) {
          console.warn(error);
        }

        break;
      default:
        console.error(`Unhandled case: ${settings.modeSelector}`);
        break;
    }
  };

  /**
   * @param {*} timerID - The UUID of the prop to check.
   * @throws An error if a prop with matching ID does not exist in `#propStatus`.
   * @returns True if the timer with the given UUID is active, false otherwise.
   */
  #propIsActive = (propID) => {
    const propMatch = this.#propStatus.find(({ id }) => id.uuid === propID);

    if (typeof propMatch === 'undefined') {
      throw new Error('Checking if a prop is active that does not exist in prop status');
    }

    return propMatch.is_active;
  };
}

if (typeof module !== 'undefined') {
  module.exports = PropAction;
}
