/* global ActionBase, StreamingEndpoint */

class StageAction extends ActionBase {
  // The stage message that is currently active on the output.
  #liveStageMessage;

  // Public readonly access of live stage message, needed for testing
  get liveStageMessage() { return this.#liveStageMessage; }

  constructor(apiManager) {
    super(apiManager);
    this.shouldPollForData = true;
    apiManager.registerCallbackForStreamingEndpoint(StreamingEndpoint.StageMessage, (message) => {
      this.#liveStageMessage = message;
      this.getSettings(); // Get settings so that icon-state can be updated
    });
    this.#liveStageMessage = '';
  }

  onDidReceiveSettings(json) {
    this.#updateIconState(json.payload.settings, json.context);
    super.onDidReceiveSettings(json);
  }

  setDefaultsInSettings(settings) {
    let didUpdateSettings = false;
    const defaults = {
      modeSelector: 'SetLayouts',
      'icon-state': 'show',
      message: '',
      stageLayoutMap: [],
    };
    Object.entries(defaults).forEach(([key, value]) => {
      if (typeof settings[key] === 'undefined') {
        settings[key] = value;
        didUpdateSettings = true;
      }
    });
    return super.setDefaultsInSettings(settings) || didUpdateSettings;
  }

  async pollForData(context, action) {
    try {
      const stageScreens = await this.apiManager.getStageScreens();
      const stageLayouts = await this.apiManager.getStageLayouts();
      const stageInfo = { availableStageScreens: stageScreens, availableStageLayouts: stageLayouts };
      ActionBase.sendToPropertyInspector(context, stageInfo, action);
    } catch (error) {
      console.warn(error);
      ActionBase.sendToPropertyInspector(context, { availableStageScreens: null, availableStageLayouts: null }, action);
    }

    super.pollForData(context, action);
  }

  async performAction(settings) {
    const modeSelection = settings.modeSelector;
    if (typeof modeSelection === 'undefined') {
      throw new Error('modeSelector not defined in settings');
    }

    const self = this;

    async function showMessage() {
      if (typeof settings.message === 'undefined') {
        throw new Error('message not defined in settings');
      }
      await self.apiManager.showStageMessage(settings.message);
    }

    async function setLayouts() {
      if (typeof settings.stageLayoutMap === 'undefined') {
        throw new Error('stageLayoutMap not defined in settings');
      }
      await self.apiManager.setStageLayouts(settings.stageLayoutMap);
    }

    switch (modeSelection) {
      case ('SetLayouts'):
        await setLayouts();
        break;
      case ('ShowStageMessage'):
        await showMessage();
        break;
      case ('HideStageMessage'):
        await this.apiManager.hideStageMessage();
        break;
      case ('ToggleStageMessage'):
        if (this.#liveStageMessage === '') {
          await showMessage();
        } else {
          await this.apiManager.hideStageMessage();
        }
        break;
      default:
        throw new Error(`Unhandled action selection: ${modeSelection}`);
    }
  }

  // *** Private Methods ***

  /**
   * Sets the appropriate `icon-state` in settings and also saves the settings if necessary.
   * @param {Object} settings - The current settings for this action instance.
   * @param {*} context - The context of this action instance.
   */
  #updateIconState = (settings, context) => {
    switch (settings.modeSelector) {
      case 'ShowStageMessage':
      case 'SetLayouts':
        ActionBase.setSetting('icon-state', 'show', settings, context);
        break;
      case 'HideStageMessage':
        ActionBase.setSetting('icon-state', 'hide', settings, context);
        break;
      case 'ToggleStageMessage':
        if (this.#liveStageMessage === '') {
          ActionBase.setSetting('icon-state', 'show', settings, context);
        } else {
          ActionBase.setSetting('icon-state', 'hide', settings, context);
        }
        break;
      default:
        console.error(`Unhandled case: ${settings.modeSelector}`);
        break;
    }
  };
}

if (typeof module !== 'undefined') {
  module.exports = StageAction;
}
