/* global ActionBase, StreamingEndpoint */

class TimelineAction extends ActionBase {
  #timelineStatus;

  // Public readonly access of timeline status, needed for testing
  get timelineStatus() { return this.#timelineStatus; }

  constructor(apiManager) {
    super(apiManager);
    this.#timelineStatus = {};

    apiManager.registerCallbackForStreamingEndpoint(StreamingEndpoint.Timeline, (timelineStatus) => {
      this.#timelineStatus = timelineStatus;
      this.getSettings(); // Get settings so that the icon updates appropriately for toggle play/stop
    });
  }

  async onDidReceiveSettings(json) {
    this.#updateIconState(json.payload.settings, json.context);
    super.onDidReceiveSettings(json);
  }

  setDefaultsInSettings(settings) {
    let didUpdateSettings = false;

    if (typeof settings.modeSelector === 'undefined') {
      settings.modeSelector = 'Play';
      didUpdateSettings = true;
    }

    if (typeof settings['icon-state'] === 'undefined') {
      settings['icon-state'] = 'play';
      didUpdateSettings = true;
    }

    return super.setDefaultsInSettings(settings) || didUpdateSettings;
  }

  async performAction(settings) {
    const modeSelection = settings.modeSelector;
    if (typeof modeSelection === 'undefined') {
      throw new Error('modeSelector not defined in settings');
    }

    switch (modeSelection) {
      case ('Play'):
        await this.apiManager.timelinePlay();
        break;
      case ('Stop'):
        await this.apiManager.timelinePause();
        break;
      case ('PlayStopTimeline'):
        if (this.#focusedTimelineIsPlaying()) {
          await this.apiManager.timelinePause();
        } else {
          await this.apiManager.timelinePlay();
        }
        break;
      case ('GoToBeginning'):
        await this.apiManager.timelineRewind();
        break;
      default:
        throw new Error(`Unhandled action selection: ${modeSelection}`);
    }
  }

  // *** Private Methods ***

  /**
   * Sets the appropriate `icon-state` in settings and also saves the settings if necessary.
   * @param {Object} settings - The current settings for this action instance.
   * @param {string} context - The context of this action instance.
   */
  #updateIconState = (settings, context) => {
    const modeSelection = settings.modeSelector;
    if (typeof modeSelection === 'undefined') {
      console.error('modeSelector not defined in settings');
      return;
    }

    switch (modeSelection) {
      case ('Play'):
        ActionBase.setSetting('icon-state', 'play', settings, context);
        break;
      case ('Stop'):
        ActionBase.setSetting('icon-state', 'stop', settings, context);
        break;
      case ('PlayStopTimeline'):
        this.#setIconStateForPlayStop(settings, context);
        break;
      case ('GoToBeginning'):
        ActionBase.setSetting('icon-state', 'goToBeginning', settings, context);
        break;
      default:
        console.error(`Unhandled case: ${modeSelection}`);
    }
  };

  /**
   * Sets the appropriate `icon-state` in settings for the play/stop action and also saves the settings if necessary.
   * @param {Object} settings - The current settings for this action instance.
   * @param {*} context - The context of this action instance.
   */
  #setIconStateForPlayStop = (settings, context) => {
    const modeSelection = settings.modeSelector;
    if (typeof modeSelection === 'undefined') {
      console.warn('modeSelector not defined in settings');
      return;
    }

    let timelineIsPlaying;

    try {
      timelineIsPlaying = this.#focusedTimelineIsPlaying();
    } catch (error) {
      console.warn(error);
    }

    if (timelineIsPlaying) {
      ActionBase.setSetting('icon-state', 'stop', settings, context);
    } else {
      ActionBase.setSetting('icon-state', 'play', settings, context);
    }
  };

  /**
   * @throws An error if a timeline status object does not exist.
   * @returns True if the focused timeline in ProPresenter is playing, false otherwise.
   */
  #focusedTimelineIsPlaying = () => {
    if (typeof this.#timelineStatus.is_running === 'undefined') {
      throw new Error('Checking playback state of focused timeline');
    }

    return this.#timelineStatus.is_running;
  };
}

if (typeof module !== 'undefined') {
  module.exports = TimelineAction;
}
