/* global ActionBase, StreamingEndpoint */

class TimerAction extends ActionBase {
  #timerStatus;

  get timerStatus() { return this.#timerStatus; }

  // Opacity is 30% in ProPresenter UI, but looks best on Stream Deck at 100%
  static get OPACITY() { return 1.0; }

  static get RUNNING_COLOR() { return `rgba(33,96,31,${TimerAction.OPACITY})`; }

  static get OVERRUN_COLOR() { return `rgba(162,36,36,${TimerAction.OPACITY}`; }

  static get EXPIRED_COLOR() { return `rgba(36,72,162,${TimerAction.OPACITY}`; }

  constructor(apiManager) {
    super(apiManager);
    this.shouldPollForData = true;
    this.#timerStatus = [];

    apiManager.registerCallbackForStreamingEndpoint(StreamingEndpoint.Timers, (status) => {
      this.#timerStatus = status;
      this.getSettings(); // Get settings so that icon-state can be updated
    });
  }

  async onDidReceiveSettings(json) {
    this.#updateTitle(json.payload.settings, json.context);
    this.#updateIconState(json.payload.settings, json.context);
    super.onDidReceiveSettings(json);
  }

  setDefaultsInSettings(settings) {
    let didUpdateSettings = false;
    const defaults = {
      modeSelector: 'StartTimer',
      'icon-state': 'start',
      shouldSetConfiguration: false,
      timerTypeSelector: 'CountdownTimer',
      incrementAmount: 0,
      countdownDuration: 0,
      countdownToTime: 12 * 60 * 60,
      countdownToTimeFormatSelector: 'am',
      elapsedStartTime: 0,
      elapsedEndTime: null,
      allowsOverrun: false,
    };

    Object.keys(defaults).forEach((key) => {
      if (typeof settings[key] === 'undefined') {
        settings[key] = defaults[key];
        didUpdateSettings = true;
      }
    });

    return super.setDefaultsInSettings(settings) || didUpdateSettings;
  }

  async pollForData(context, action) {
    try {
      const timers = await this.apiManager.getTimers();
      ActionBase.sendToPropertyInspector(context, { availableTimers: timers }, action);
    } catch (error) {
      console.warn(error);
      ActionBase.sendToPropertyInspector(context, { availableTimers: null }, action);
    }

    super.pollForData(context, action);
  }

  async performAction(settings) {
    const timerID = settings.timerSelector;
    const modeSelection = settings.modeSelector;

    if (typeof timerID === 'undefined') {
      throw new Error('timerSelector not defined in settings');
    } else if (typeof modeSelection === 'undefined') {
      throw new Error('modeSelector not defined in settings');
    }

    let configuration = null;
    if (settings.shouldSetConfiguration) {
      configuration = TimerAction.#getConfiguration(settings);
    }

    switch (modeSelection) {
      case ('StartTimer'):
        await this.apiManager.startTimer(timerID, configuration);
        break;
      case ('StopTimer'):
        await this.apiManager.stopTimer(timerID, configuration);
        break;
      case ('StartStopTimer'):
        await this.startStopTimer(timerID);
        break;
      case ('ResetTimer'):
        await this.apiManager.resetTimer(timerID, configuration);
        break;
      case ('IncrementTimer'):
        await this.incrementTimer(timerID, settings);
        break;
      default:
        throw new Error(`Unhandled action selection: ${modeSelection}`);
    }
  }

  async startStopTimer(timerID) {
    const timerState = this.#getTimerState(timerID);
    switch (timerState) {
      case 'running':
        await this.apiManager.stopTimer(timerID);
        this.#setTimerState(timerID, 'stopped'); // Set state immediately to avoid lag
        break;
      case 'overrun': // 7.9 used this case for 'overrunning' and 'overran'
      case 'overrunning':
        await this.apiManager.stopTimer(timerID);
        this.#setTimerState(timerID, 'overran');
        break;
      case 'overran':
        await this.apiManager.startTimer(timerID);
        this.#setTimerState(timerID, 'overrunning');
        break;
      case 'stopped':
        await this.apiManager.startTimer(timerID);
        this.#setTimerState(timerID, 'running');
        break;
      case 'complete':
        await this.apiManager.resetTimer(timerID);
        this.#setTimerState(timerID, 'stopped');
        break;
      default:
        break;
    }
    this.getSettings();
  }

  async incrementTimer(timerID, settings) {
    if (typeof settings.incrementAmount === 'undefined') {
      throw new Error('increment amount not defined in settings');
    }
    await this.apiManager.incrementTimer(timerID, settings.incrementAmount);
  }

  static #getConfiguration = (settings) => {
    const configuration = {
      id: {
        uuid: settings.timerSelector,
      },
      allows_overrun: settings.allowsOverrun,
    };

    switch (settings.timerTypeSelector) {
      case 'CountdownTimer':
        configuration.countdown = {
          duration: settings.countdownDuration,
        };
        break;
      case 'CountdownToTime':
        configuration.count_down_to_time = {
          time_of_day: settings.countdownToTime,
          period: settings.countdownToTimeFormatSelector,
        };
        break;
      case 'ElapsedTime':
        configuration.elapsed = {
          start_time: settings.elapsedStartTime,
          end_time: settings.elapsedEndTime,
        };
        break;
      default:
        throw new Error(`Unhandled timer type selection: ${settings.timerTypeSelector}`);
    }
    return configuration;
  };

  /**
   * Sets the appropriate `icon-state` in settings and also saves the settings if necessary.
   * @param {Object} settings - The current settings for this action instance.
   * @param {*} context - The context of this action instance.
   */
  #updateIconState = (settings, context) => {
    switch (settings.modeSelector) {
      case 'StartTimer':
        ActionBase.setSetting('icon-state', 'start', settings, context);
        break;
      case 'StopTimer':
        ActionBase.setSetting('icon-state', 'stop', settings, context);
        break;
      case 'StartStopTimer':
        this.#setIconStateAndColorForStartStop(settings, context);
        break;
      case ('ResetTimer'):
        ActionBase.setSetting('icon-state', 'reset', settings, context);
        break;
      case ('IncrementTimer'):
        ActionBase.setSetting('icon-state', 'increment', settings, context);
        break;
      default:
        console.error(`Unhandled case: ${settings.modeSelector}`);
        break;
    }
  };

  /**
   * Sets the title to the current time of the timer when the 'StartStopTimer' action is selected.
   * @param {Object} settings - The current settings for this action instance.
   * @param {*} context - The context of this action instance.
   */
  #updateTitle = (settings, context) => {
    if (settings.modeSelector === 'StartStopTimer' && typeof settings.timerSelector !== 'undefined') {
      let { time } = this.#timerStatusForID(settings.timerSelector);
      if (typeof time === 'undefined') {
        console.warn(`No status for timer ID: ${settings.timerSelector}`);
        return;
      }
      time = time.replace(/^(00:)/, ''); // Strip leading zeros
      time = time.replace(/^(-00:)/, '-'); // Strip leading zeros for negative numbers
      time = time.replace(/(\.[0-9][0-9])$/, ''); // Strip trailing milliseconds
      ActionBase.setTitle(context, time);
    } else {
      ActionBase.clearTitle(context);
    }
  };

  /**
   * Sets the appropriate `icon-state` in settings for the start/stop action and also saves the settings if necessary.
   * Also sets the `stateColor` in settings which does not need to be persisted.
   * @param {Object} settings - The current settings for this action instance.
   * @param {*} context - The context of this action instance.
   */
  #setIconStateAndColorForStartStop = (settings, context) => {
    if (typeof settings.timerSelector === 'undefined') {
      console.warn('timerSelector is not defined in settings');
      return;
    }

    let timerState;

    try {
      timerState = this.#getTimerState(settings.timerSelector);
    } catch (error) {
      console.warn(error);
    }

    switch (timerState) {
      case 'running':
        ActionBase.setSetting('icon-state', 'stop', settings, context);
        settings.stateColor = TimerAction.RUNNING_COLOR;
        break;
      case 'overrun': // 7.9 used this case for 'overrunning' and 'overran'
      case 'overrunning':
        ActionBase.setSetting('icon-state', 'stop', settings, context);
        settings.stateColor = TimerAction.OVERRUN_COLOR;
        break;
      case 'overran':
        ActionBase.setSetting('icon-state', 'start', settings, context);
        settings.stateColor = TimerAction.OVERRUN_COLOR;
        break;
      case 'stopped':
        ActionBase.setSetting('icon-state', 'start', settings, context);
        settings.stateColor = null;
        break;
      case 'complete':
        ActionBase.setSetting('icon-state', 'reset', settings, context);
        settings.stateColor = TimerAction.EXPIRED_COLOR;
        break;
      default:
        console.error(`Unhandled timer state case: ${timerState}`);
        break;
    }
  };

  /**
   * @param {*} timerID - The UUID of the timer to check.
   * @throws An error if a timer with matching ID does not exist in the timer status.
   * @returns True if the timer with the given UUID is active, false otherwise.
   */
  #getTimerState = (timerID) => {
    const timerStatus = this.#timerStatusForID(timerID);

    if (typeof timerStatus === 'undefined') {
      throw new Error('Checking if a timer is active that does not exist in timer status');
    }
    return timerStatus.state;
  };

  /**
   * @param {*} timerID - The UUID of the timer to toggle.
   * @throws An error if a timer with matching ID does not exist in the timer status.
   */
  #setTimerState = (timerID, state) => {
    const timerStatus = this.#timerStatusForID(timerID);

    if (typeof timerStatus === 'undefined') {
      throw new Error('Trying to set state of a timer that does not exist in timer status');
    }

    timerStatus.state = state;
  };

  /**
   * @param {*} timerID - The UUID of the timer to look for in timer status.
   * @returns The status of the timer if found, otherwise `undefined`.
   */
  #timerStatusForID = (timerID) => this.#timerStatus.find(({ id }) => id.uuid === timerID);
}

if (typeof module !== 'undefined') {
  module.exports = TimerAction;
}
