/* global ActionBase, StreamingEndpoint */

class TransportAction extends ActionBase {
  #layerStatus;

  // Public readonly access of layer status, needed for testing
  get layerStatus() { return this.#layerStatus; }

  constructor(apiManager) {
    super(apiManager);
    this.#layerStatus = {};

    apiManager.registerCallbackForStreamingEndpoint(StreamingEndpoint.TransportAnnouncement, (status) => {
      this.#layerStatus.announcement = status;
      this.getSettings(); // Get settings so that icon-state can be updated
    });
    apiManager.registerCallbackForStreamingEndpoint(StreamingEndpoint.TransportAudio, (status) => {
      this.#layerStatus.audio = status;
      this.getSettings(); // Get settings so that icon-state can be updated
    });
    apiManager.registerCallbackForStreamingEndpoint(StreamingEndpoint.TransportPresentation, (status) => {
      this.#layerStatus.presentation = status;
      this.getSettings(); // Get settings so that icon-state can be updated
    });
  }

  async onDidReceiveSettings(json) {
    this.#updateIconState(json.payload.settings, json.context);
    TransportAction.#setTitleForState(json.payload.settings, json.context);
    super.onDidReceiveSettings(json);
  }

  onWillAppear(json) {
    super.onWillAppear(json);
    TransportAction.#setTitleForState(json.payload.settings, json.context);
  }

  setDefaultsInSettings(settings) {
    let didUpdateSettings = false;

    if (typeof settings['icon-state'] === 'undefined') {
      settings['icon-state'] = 'play';
      didUpdateSettings = true;
    }

    if (typeof settings.modeSelector === 'undefined') {
      settings.modeSelector = 'Play';
      didUpdateSettings = true;
    }

    if (typeof settings.transportLayerSelector === 'undefined') {
      settings.transportLayerSelector = 'Presentation';
      didUpdateSettings = true;
    }

    if (typeof settings.time === 'undefined') {
      settings.time = 15;
      didUpdateSettings = true;
    }

    return super.setDefaultsInSettings(settings) || didUpdateSettings;
  }

  async performAction(settings) {
    const layerSelection = settings.transportLayerSelector; // User readable

    if (typeof layerSelection === 'undefined') {
      throw new Error('transportLayerSelector not defined in settings');
    }

    const layer = TransportAction.#apiLayerForLayerSelection(layerSelection); // What is actually sent to the API
    const { time } = settings;

    const modeSelection = settings.modeSelector;

    if (typeof modeSelection === 'undefined') {
      throw new Error('modeSelector not defined in settings');
    }

    switch (modeSelection) {
      case ('Play'):
        await this.apiManager.transportPlay(layer);
        break;
      case ('Pause'):
        await this.apiManager.transportPause(layer);
        break;
      case ('PlayPause'): {
        const layerStatus = this.#layerStatus[layer];
        if (typeof layerStatus === 'undefined') {
          throw new Error(`No status for layer: ${layer}`);
        }

        if (layerStatus.is_playing) {
          await this.apiManager.transportPause(layer);
        } else {
          await this.apiManager.transportPlay(layer);
        }
        break;
      }
      case ('SkipForward'):
        await this.apiManager.transportSkipForward(layer, time);
        break;
      case ('SkipBackward'):
        await this.apiManager.transportSkipBackward(layer, time);
        break;
      case ('GoToEnd'):
        await this.apiManager.transportGoToEnd(layer, time);
        break;
      default:
        throw new Error(`Unhandled mode selection: ${modeSelection}`);
    }
  }

  // *** Private Methods ***

  /**
   * Sets the appropriate `icon-state` in settings and also saves the settings if necessary.
   * @param {Object} settings - The current settings for this action instance.
   * @param {*} context - The context of this action instance.
   */
  #updateIconState = (settings, context) => {
    switch (settings.modeSelector) {
      case 'Play':
        ActionBase.setSetting('icon-state', 'play', settings, context);
        break;
      case 'Pause':
        ActionBase.setSetting('icon-state', 'pause', settings, context);
        break;
      case 'PlayPause':
        this.#setIconStateForPlayPause(settings, context);
        break;
      case 'SkipForward':
        ActionBase.setSetting('icon-state', 'skipForward', settings, context);
        break;
      case 'SkipBackward':
        ActionBase.setSetting('icon-state', 'skipBackward', settings, context);
        break;
      case 'GoToEnd':
        ActionBase.setSetting('icon-state', 'goToEnd', settings, context);
        break;
      default:
        console.error(`Unhandled case: ${settings.modeSelector}`);
        break;
    }
  };

  /**
   * Sets the appropriate `icon-state` in settings for the play/pause action and also saves the settings if necessary.
   * @param {Object} settings - The current settings for this action instance.
   * @param {*} context - The context of this action instance.
   */
  #setIconStateForPlayPause = (settings, context) => {
    const layerSelection = settings.transportLayerSelector;
    if (typeof layerSelection === 'undefined') {
      console.warn('transportLayerSelector is not defined in settings');
      return;
    }

    const layer = TransportAction.#apiLayerForLayerSelection(layerSelection);
    const layerStatus = this.#layerStatus[layer];
    if (typeof layerStatus === 'undefined') {
      console.warn(`No status for layer: ${layer}`);
      // We still want to set the icon to `play` in the case that there is no connection to ProPresenter
      ActionBase.setSetting('icon-state', 'play', settings, context);
      return;
    }

    if (layerStatus.is_playing && layerStatus.duration !== 0) {
      ActionBase.setSetting('icon-state', 'pause', settings, context);
    } else {
      ActionBase.setSetting('icon-state', 'play', settings, context);
    }
  };

  /**
   * @param {string} layerSelection - The layer selection from the UI.
   * @returns {string} The api layer string that corresponds with the layer selection
   */
  static #apiLayerForLayerSelection = (layerSelection) => {
    switch (layerSelection) {
      case ('Presentation'):
        return 'presentation';
      case ('Announcements'):
        return 'announcement';
      case ('Audio'):
        return 'audio';
      default:
        return 'presentation';
    }
  };

  /**
     * Sets the title to the time remaining if applicable.
     *
     * @param {Object} settings - The settings to use when determining the title.
     * @param {string} context - The context UUID of this action instance.
     */
  static #setTitleForState = (settings, context) => {
    if (settings['icon-state'] !== 'play' && settings['icon-state'] !== 'pause') {
      ActionBase.setTitle(context, settings.time);
    } else {
      ActionBase.clearTitle(context);
    }
  };
}

if (typeof module !== 'undefined') {
  module.exports = TransportAction;
}
