/* global IPV4_REGEX_PATTERN, $ */

// The amount of time in ms to wait for a response when establishing connection before timing out.
const establishConnectionTimeout = 5000;

// Abort controller for cancelling the fetch request when establishing connection.
let abortController = null;

// Enum used to represent the different screens used for setup.
// Each setup "screen" uses a div in the HTML and so we can switch screens by hiding and showing divs.
class SetupScreen {
  static AddConnection = new SetupScreen('add-connection');

  static EstablishConnection = new SetupScreen('establish-connection');

  static ConnectionError = new SetupScreen('connection-error');

  static All = [this.AddConnection, this.EstablishConnection, this.ConnectionError];

  constructor(name) {
    this.name = name;
  }

  show() {
    $(`#${this.name}`).show();
  }

  hide() {
    $(`#${this.name}`).hide();
  }
}

// Hides any currently showing screen and shows the screen passed in.
function switchToScreen(setupScreen) {
  SetupScreen.All.forEach((screen) => {
    screen.hide();
  });

  setupScreen.show();
}

// Ensure ip input text is a correctly formatted valid IPv4 address
function validateIPInput(ipInput) {
  // Uses IPv4 regex pattern to ensure user entered a valid ip address, does not work with ipv6
  return new RegExp(IPV4_REGEX_PATTERN).test(ipInput);
}

// Ensure port input text is a valid integer and within the correct allowed port range
function validatePortInput(portInput) {
  // Convert the entered string port number to an integer
  const portNumber = parseInt(portInput, 10);
  return !Number.isNaN(portNumber) && (portNumber >= 1024 && portNumber <= 65535);
}

// Called after timing out when trying to establish connection.
function abortEstablishConnection() {
  if (abortController === null) {
    return;
  }
  abortController.abort();
  switchToScreen(SetupScreen.ConnectionError);
}

async function establishConnection(ip, port) {
  // TODO: Replace the heartbeat endpoint with the final public one
  const url = `http://${ip}:${port}/version`;

  setTimeout(() => abortEstablishConnection(), establishConnectionTimeout);
  abortController = new AbortController();
  let response;

  try {
    response = await fetch(url, {
      method: 'GET',
      signal: abortController.signal,
    });
  } catch (error) {
    console.warn(`Error establishing connection: ${error}`);
    if (error.name !== 'AbortError') {
      switchToScreen(SetupScreen.ConnectionError);
    }
    abortController = null;
    return;
  }

  if (response.ok) {
    const json = await response.json();
    const { name } = json;

    window.opener.document.dispatchEvent(new CustomEvent('saveConnection', {
      detail: {
        name,
        ip,
        port,
      },
    }));
    window.close();
  } else {
    switchToScreen(SetupScreen.ConnectionError);
  }
}

function validateConnectionInfo() {
  const ipInput = document.getElementById('ip-address-input');
  const portInput = document.getElementById('port-number-input');
  const ip = ipInput.value.trim();
  const port = portInput.value.trim();
  const isValidIPInput = validateIPInput(ip);
  const isValidPortInput = validatePortInput(port);

  // Shows browser validity error over ip input field and returns out
  if (!isValidIPInput) {
    ipInput.setCustomValidity('Invalid IPv4 address format.');
    ipInput.reportValidity();
    ipInput.focus();
    return;
  }

  // Shows browser validity error over port input field and returns out
  if (!isValidPortInput) {
    portInput.setCustomValidity('Port number should be between 1024 and 65535.');
    portInput.reportValidity();
    portInput.focus();
    return;
  }

  // We have valid ip and port input, so switch the UI to the "Establishing Connection" UI and make network call for status
  if (isValidIPInput && isValidPortInput) {
    switchToScreen(SetupScreen.EstablishConnection);
    establishConnection(ip, port);
  }
}

// Runs when setup window loads
window.onload = () => {
  // Set cursor to ip input field when the setup window loads
  $('#ip-address-input').focus();

  // Adds a listener for whenever the ip input field changes, which gets called when the field loses focus not on each keystroke
  $('#ip-address-input').change(() => {
    const ip = $('#ip-address-input').val().trim();

    // Ensure input has some value else revert back to placeholder style
    if (ip !== '') {
      if (validateIPInput(ip)) {
        $('#ip-address-input').css('border-color', 'green');
      } else {
        $('#ip-address-input').css('border-color', 'red');
      }
    } else {
      // Changes the input text field back to the placeholder color when field is empty
      $('#ip-address-input').css('border-color', '#8d8d8d');
    }
  });

  // Adds a listener for whenever the port number input field changes, which gets called when the field loses focus not on each keystroke
  $('#port-number-input').change(() => {
    const port = $('#port-number-input').val().trim();

    // Ensure input has some value else revert back to placeholder style
    if (port !== '') {
      if (validatePortInput(port)) {
        $('#port-number-input').css('border-color', 'green');
      } else {
        $('#port-number-input').css('border-color', 'red');
      }
    } else {
      // Changes the input text field back to the placeholder color when field is empty
      $('#port-number-input').css('border-color', '#8d8d8d');
    }
  });

  $('#continue-button').on('click', () => {
    validateConnectionInfo();
  });

  // Adds a listener for the cancel button to close the window when cancel is clicked
  $('#cancel-add').on('click', () => {
    window.close();
  });

  $('#cancel-establish').on('click', () => {
    if (abortController === null) {
      return;
    }
    abortController.abort();
    switchToScreen(SetupScreen.AddConnection);
  });

  $('#error-ok').on('click', () => {
    switchToScreen(SetupScreen.AddConnection);
  });
};
