/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach, linkEnabled */
const assert = require('assert');
const APIManager = require('../js/APIManager');

function test() {
  // *** Setup ***

  let apiManager;
  const testBaseURL = 'http://localhost:8080';

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    global.linkEnabled = true;
    apiManager = new APIManager(testBaseURL);
    done();
  });

  // *** Tests ***

  it('Should initialize a new instance of Api Server', () => {
    assert(apiManager != null);
  });

  it('Should set the baseurl', () => {
    assert(apiManager.baseUrl === testBaseURL);
  });

  it('Should throw an error when attempting to make a call with no base url', async () => {
    const noBaseApiManager = new APIManager('');
    await assert.rejects(noBaseApiManager.showStageMessage('Hello World'), Error);
  });

  // *** Audio Action ***

  it('Should return playlists when getting audio playlists', async () => {
    const playlists = await apiManager.getAudioPlaylists();
    assert(playlists.length === 1);
  });

  it('Should return items when getting audio items in playlist', async () => {
    const items = await apiManager.getAudioItemsInPlaylist('PLAYLIST_ID');
    assert(items.length === 2);
  });

  it('Should return a successful response when triggering an audio playlist', async () => {
    const response = await apiManager.triggerAudio('PLAYLIST_ID', null);
    assert(response.status === 204);
  });

  it('Should return a successful response when triggering an item within an audio playlist', async () => {
    const response = await apiManager.triggerAudio('PLAYLIST_WITH_ITEMS_ID', 'ITEMS_ID');
    assert(response.status === 204);
  });

  it('Should throw an error when triggering audio with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.triggerAudio('PLAYLIST_ID', null), Error);
  });

  // *** Stage Action ***

  it('Should return screens when getting stage screens', async () => {
    const screens = await apiManager.getStageScreens();
    assert(screens.length === 2);
  });

  it('Should return layouts when getting stage layouts', async () => {
    const screens = await apiManager.getStageLayouts();
    assert(screens.length === 2);
  });

  it('Should return a successful response when setting stage layouts', async () => {
    const response = await apiManager.setStageLayouts([]);
    assert(response.status === 204);
  });

  it('Should throw an error when setting stage layouts with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.setStageLayouts([]), Error);
  });

  it('Should return a successful response when showing a stage message', async () => {
    const response = await apiManager.showStageMessage('Hello World');
    assert(response.status === 204);
  });

  it('Should throw an error when showing a stage message with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.showStageMessage('Hello World'), Error);
  });

  it('Should return a successful response when hiding a stage message', async () => {
    const response = await apiManager.hideStageMessage();
    assert(response.status === 204);
  });

  it('Should throw an error when hiding a stage message with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.hideStageMessage, Error);
  });

  // *** Slide Action ***

  it('Should return a successful response when triggering the next slide', async () => {
    const response = await apiManager.triggerNextSlide();
    assert(response.status === 204);
  });

  it('Should throw an error when triggering the next slide with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.triggerNextSlide(), Error);
  });

  it('Should return a successful response when triggering the previous slide', async () => {
    const response = await apiManager.triggerPreviousSlide();
    assert(response.status === 204);
  });

  it('Should throw an error when triggering the previous slide with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.triggerPreviousSlide(), Error);
  });

  // *** Find My Mouse Action ***

  it('Should return a successful response when triggering find my mouse', async () => {
    const response = await apiManager.findMyMouse();
    assert(response.status === 204);
  });

  it('Should throw an error when triggering find my mouse with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.findMyMouse(), Error);
  });

  // *** Capture Action ***

  it('Should return a successful response when starting capture', async () => {
    const response = await apiManager.startCapture();
    assert(response.status === 204);
  });

  it('Should throw an error when starting capture with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.startCapture(), Error);
  });

  it('Should return a successful response when stopping capture', async () => {
    const response = await apiManager.stopCapture();
    assert(response.status === 204);
  });

  it('Should throw an error when stopping capture with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.stopCapture(), Error);
  });

  // *** Timer Action ***

  it('Should return timers when getting timers', async () => {
    const timers = await apiManager.getTimers();
    assert(timers.length === 2);
  });

  it('Should throw an error when getting timers with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.getTimers(), Error);
  });

  it('Should return a successful response when starting a timer', async () => {
    const response = await apiManager.startTimer('1');
    assert(response.status === 204);
  });

  it('Should return a successful response when starting a timer with a configuration', async () => {
    const configuration = {
      id: {
        uuid: '942C3FC3-C4B2-44F7-A55D-4CC913BB8A5D',
        name: 'Timer 1',
        index: 0,
      },
      allows_overrun: true,
      countdown: {
        duration: 300,
      },
    };
    const response = await apiManager.startTimer('1', configuration);
    assert(response.status === 204);
  });

  it('Should throw an error wen starting a timer with an invalid configuration', async () => {
    await assert.rejects(apiManager.startTimer('1', {}), Error);
  });

  it('Should throw an error when starting a timer with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.startTimer('1'), Error);
  });

  it('Should return a successful response when stopping a timer', async () => {
    const response = await apiManager.stopTimer('1');
    assert(response.status === 204);
  });

  it('Should return a successful response when stopping a timer with a configuration', async () => {
    const configuration = {
      id: {
        uuid: '942C3FC3-C4B2-44F7-A55D-4CC913BB8A5D',
        name: 'Timer 1',
        index: 0,
      },
      allows_overrun: true,
      countdown: {
        duration: 300,
      },
    };
    const response = await apiManager.stopTimer('1', configuration);
    assert(response.status === 204);
  });

  it('Should throw an error wen stopping a timer with an invalid configuration', async () => {
    await assert.rejects(apiManager.stopTimer('1', {}), Error);
  });

  it('Should throw an error when stopping a timer with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.stopTimer('1'), Error);
  });

  it('Should return a successful response when resetting a timer', async () => {
    const response = await apiManager.resetTimer('1');
    assert(response.status === 204);
  });

  it('Should throw an error when stopping a timer with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.resetTimer('1'), Error);
  });

  it('Should return a successful response when incrementing a timer', async () => {
    const response = await apiManager.incrementTimer('1', 60);
    assert(response.status === 204);
  });

  it('Should throw an error when incrementing a timer with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.incrementTimer('1', 60), Error);
  });

  // *** Look Action ***

  it('Should return looks when getting looks', async () => {
    const looks = await apiManager.getLooks();
    assert(looks.length === 2);
  });

  it('Should throw an error when getting looks with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.getLooks(), Error);
  });

  it('Should return a successful response when triggering a look', async () => {
    const response = await apiManager.triggerLook('1');
    assert(response.status === 204);
  });

  it('Should throw an error when triggering a look with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.triggerLook('1'), Error);
  });

  // *** Macro Action ***

  it('Should return macros when getting macros', async () => {
    const macros = await apiManager.getMacros();
    assert(macros.length === 2);
  });

  it('Should throw an error when getting macros with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.getMacros(), Error);
  });

  it('Should return a successful response when triggering a macro', async () => {
    const response = await apiManager.triggerMacro('1');
    assert(response.status === 204);
  });

  it('Should throw an error when triggering a macro with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.triggerMacro('1'), Error);
  });

  // *** Transport Action ***

  it('Should return a successful response when playing on a transport layer', async () => {
    const response = await apiManager.transportPlay('presentation');
    assert(response.status === 204);
  });

  it('Should throw an error when playing on an invalid transport layer', async () => {
    await assert.rejects(apiManager.transportPlay('invalid'), Error);
  });

  it('Should throw an error when playing and link is disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.transportPlay('presentation'), Error);
  });

  it('Should return a successful response when pausing on a transport layer', async () => {
    const response = await apiManager.transportPause('presentation');
    assert(response.status === 204);
  });

  it('Should throw an error when pausing on an invalid transport layer', async () => {
    await assert.rejects(apiManager.transportPause('invalid'), Error);
  });

  it('Should throw an error when pausing and link is disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.transportPause('presentation'), Error);
  });

  it('Should return a successful response when skipping backward on a transport layer', async () => {
    const response = await apiManager.transportSkipBackward('presentation', 15);
    assert(response.status === 204);
  });

  it('Should throw an error when skipping backward on an invalid transport layer', async () => {
    await assert.rejects(apiManager.transportSkipBackward('invalid', 15), Error);
  });

  it('Should throw an error when skipping backward with an invalid time', async () => {
    await assert.rejects(apiManager.transportSkipBackward('presentation', 'foobar'), Error);
  });

  it('Should throw an error when skipping backward and link is disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.transportSkipBackward('presentation', 15), Error);
  });

  it('Should return a successful response when skipping forward on a transport layer', async () => {
    const response = await apiManager.transportSkipForward('presentation', 15);
    assert(response.status === 204);
  });

  it('Should throw an error when skipping forward on an invalid transport layer', async () => {
    await assert.rejects(apiManager.transportSkipForward('invalid', 15), Error);
  });

  it('Should throw an error when skipping forward with an invalid time', async () => {
    await assert.rejects(apiManager.transportSkipForward('presentation', 'foobar'), Error);
  });

  it('Should throw an error when skipping forward and link is disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.transportSkipForward('presentation', 15), Error);
  });

  it('Should return a successful response when going to end on a transport layer', async () => {
    const response = await apiManager.transportGoToEnd('presentation', 15);
    assert(response.status === 204);
  });

  it('Should throw an error when going to end on an invalid transport layer', async () => {
    await assert.rejects(apiManager.transportGoToEnd('invalid', 15), Error);
  });

  it('Should throw an error when going to end with an invalid time', async () => {
    await assert.rejects(apiManager.transportGoToEnd('presentation', 'foobar'), Error);
  });

  it('Should throw an error when going to end and link is disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.transportGoToEnd('presentation', 15), Error);
  });

  // *** Prop Action ***

  it('Should return props when getting props', async () => {
    const props = await apiManager.getProps();
    assert(props.length === 3);
  });

  it('Should throw an error when getting props with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.getProps(), Error);
  });

  it('Should return a successful response when triggering a prop', async () => {
    const response = await apiManager.triggerProp('1');
    assert(response.status === 204);
  });

  it('Should throw an error when triggering a prop with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.triggerProp('1'), Error);
  });

  it('Should return a successful response when clearing a prop', async () => {
    const response = await apiManager.clearProp('1');
    assert(response.status === 204);
  });

  it('Should throw an error when clearing a prop with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.clearProp('1'), Error);
  });

  // *** Group Action ***

  it('Should return groups when getting groups', async () => {
    const groups = await apiManager.getGroups();
    assert(groups.length === 2);
  });

  it('Should throw an error when getting groups with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.getGroups(), Error);
  });

  it('Should return a successful response when triggering a group', async () => {
    const response = await apiManager.triggerGroup('1');
    assert(response.status === 204);
  });

  it('Should throw an error when triggering a group with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.triggerGroup('1'), Error);
  });

  // *** Clear Action ***

  it('Should return clear groups when getting clear groups', async () => {
    const groups = await apiManager.getClearGroups();
    assert(groups.length === 5);
  });

  it('Should throw an error when getting clear groups with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.getClearGroups(), Error);
  });

  it('Should return a successful response when triggering a clear group', async () => {
    const response = await apiManager.triggerClearGroup('1');
    assert(response.status === 204);
  });

  it('Should throw an error when triggering a clear group with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.triggerClearGroup('1'), Error);
  });

  it('Should return a successful response when clearing a layer', async () => {
    const response = await apiManager.triggerClearLayer('audio');
    assert(response.status === 204);
  });

  it('Should throw an error when clearing a layer with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.triggerClearLayer('1'), Error);
  });

  it('Should throw an error when clearing an invalid layer', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.triggerClearLayer('invalid'), Error);
  });

  // *** Timeline Action ***

  it('Should return a successful response when playing a timeline', async () => {
    const response = await apiManager.timelinePlay();
    assert(response.status === 204);
  });

  it('Should throw an error when playing a timeline with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.timelinePlay(), Error);
  });

  it('Should return a successful response when pausing a timeline', async () => {
    const response = await apiManager.timelinePause();
    assert(response.status === 204);
  });

  it('Should throw an error when pausing a timeline with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.timelinePause(), Error);
  });

  it('Should return a successful response when rewinding a timeline', async () => {
    const response = await apiManager.timelineRewind();
    assert(response.status === 204);
  });

  it('Should throw an error when rewinding a timeline with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.timelineRewind(), Error);
  });

  // *** Video Input Action ***

  it('Should return video inputs when getting video inputs', async () => {
    const macros = await apiManager.getVideoInputs();
    assert(macros.length === 2);
  });

  it('Should throw an error when getting video inputs with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.getVideoInputs(), Error);
  });

  it('Should return a successful response when triggering a video input', async () => {
    const response = await apiManager.triggerVideoInput('1');
    assert(response.status === 204);
  });

  it('Should throw an error when triggering a video input with link disabled', async () => {
    global.linkEnabled = false;
    await assert.rejects(apiManager.triggerVideoInput('1'), Error);
  });
}

module.exports = test;
