/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, Utils, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const ActionBase = require('../../js/actions/ActionBase');

function test() {
  // *** Setup ***

  let apiManager;
  let action;
  let findMyMouseJSON;

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new ActionBase(apiManager);
    findMyMouseJSON = FakeStreamDeck.generateJSON(Constants.FIND_MY_MOUSE_ACTION_ID);
    done();
  });

  // *** Tests ***

  it('Should add instance to visible instances `onWillAppear`', () => {
    action.onWillAppear(findMyMouseJSON);
    assert(action.visibleInstances[findMyMouseJSON.context] === findMyMouseJSON.context);
  });

  it('Should set defaults and save settings `onWillAppear`', () => {
    const spyOnSaveSettings = sinon.spy($SD.api, 'setSettings');

    action.onWillAppear(findMyMouseJSON);

    assert(findMyMouseJSON.payload.settings['icon-background-color'] === null);
    assert(findMyMouseJSON.payload.settings['icon-tint'] === 'lightIcon');
    assert(findMyMouseJSON.payload.settings.actionColor === null);
    assert(spyOnSaveSettings.calledWith(findMyMouseJSON.context, findMyMouseJSON.payload.settings));
  });

  it('Should not set defaults in settings if not required', () => {
    const settings = {
      'icon-background-color': 'red',
      'icon-tint': 'lightIcon',
      actionColor: 'blue',
    };

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === false);
    assert(settings['icon-background-color'] === 'red');
    assert(settings['icon-tint'] === 'lightIcon');
    assert(settings.actionColor === 'blue');
  });

  it('Should not save settings `onWillAppear` if not required', () => {
    const spyOnSaveSettings = sinon.spy($SD.api, 'setSettings');

    const settings = {
      'icon-background-color': 'red',
      'icon-tint': 'lightIcon',
      actionColor: 'blue',
    };

    findMyMouseJSON.payload.settings = settings;

    action.onWillAppear(findMyMouseJSON);

    assert(spyOnSaveSettings.notCalled);
  });

  it('Should get global settings `onWillAppear`', () => {
    const spyOnGetGlobalSettings = sinon.spy($SD.api, 'getGlobalSettings');

    action.onWillAppear(findMyMouseJSON);

    assert(spyOnGetGlobalSettings.calledWith($SD.uuid));
  });

  it('Should set image `onWillAppear`', () => {
    const spyOnLoadImage = sinon.spy(Utils, 'loadImage');
    const spyOnSetImage = sinon.spy($SD.api, 'setImage');

    action.onWillAppear(findMyMouseJSON);

    assert(spyOnLoadImage.calledWith('./images/key-icons/findMyMouse/findMyMouseKey_Light@2x.png'));
    assert(spyOnSetImage.called);
  });

  it('Should set image `onWillAppear` to first valid state when `icon-state` is invalid', () => {
    const spyOnLoadImage = sinon.spy(Utils, 'loadImage');
    const spyOnSetImage = sinon.spy($SD.api, 'setImage');

    const slideJSON = FakeStreamDeck.generateJSON(Constants.SLIDE_ACTION_ID, { 'icon-state': 'invalid' });
    action.onWillAppear(slideJSON);

    assert(spyOnLoadImage.calledWith('./images/key-icons/slide/slideNextKey_Light@2x.png'));
    assert(spyOnSetImage.called);
  });

  it('Should delete instance from visible instances `onWillDisppear`', () => {
    action.onWillAppear(findMyMouseJSON);
    action.onWillDisappear(findMyMouseJSON);
    assert(typeof action.visibleInstances[findMyMouseJSON.context] === 'undefined');
  });

  it('Should start polling `onPropertyInspectorDidAppear` and stop polling `onPropertyInspectorDidDisappear`', () => {
    action.shouldPollForData = true;

    action.onPropertyInspectorDidAppear(findMyMouseJSON);

    assert(action.isPolling === true);
    assert(action.pollingTimeout !== null);

    action.onPropertyInspectorDidDisappear(findMyMouseJSON);

    assert(action.isPolling === false);
    assert(action.pollingTimeout === null);
  });

  it('Should not start polling `onPropertyInspectorDidAppear` if `shouldPollForData` is false', () => {
    action.shouldPollForData = false;

    action.onPropertyInspectorDidAppear(findMyMouseJSON);

    assert(action.isPolling === false);
    assert(action.pollingTimeout === null);
  });

  it('Should get settings for each visible instance when calling `getSettings`', () => {
    const slideJSON = FakeStreamDeck.generateJSON(Constants.SLIDE_ACTION_ID);
    action.onWillAppear(findMyMouseJSON);
    action.onWillAppear(slideJSON);
    const spyOnGetSettings = sinon.spy($SD.api, 'getSettings');

    action.getSettings();

    assert(spyOnGetSettings.calledWith(findMyMouseJSON.context));
    assert(spyOnGetSettings.calledWith(slideJSON.context));
  });

  it('Should save settings when a setting changed using `setSetting`', () => {
    const settings = { someSetting: 'Hello World' };
    const spyOnSaveSettings = sinon.spy($SD.api, 'setSettings');

    ActionBase.setSetting('someSetting', 'Something New', settings, 'context');

    assert(spyOnSaveSettings.calledWith('context', settings));
  });

  it('Should not save settings when a setting did not change using `setSetting`', () => {
    const settings = { someSetting: 'Hello World' };
    const spyOnSaveSettings = sinon.spy($SD.api, 'setSettings');

    ActionBase.setSetting('someSetting', 'Hello World', settings, 'context');

    assert(spyOnSaveSettings.notCalled);
  });
}

module.exports = test;
