/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const AudioAction = require('../../js/actions/AudioAction');

function test() {
  let apiManager;
  let action;

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new AudioAction(apiManager);
    done();
  });

  it('Should initialize a new AudioAction instance', () => {
    assert(action != null);
  });

  it('Should trigger selected audio playlist and item', async () => {
    const json = FakeStreamDeck.generateJSON(Constants.AUDIO_ACTION_ID, { audioPlaylistSelector: '123', audioItemSelector: '456' });
    const spy = sinon.spy(apiManager, 'triggerAudio');

    action.performAction(json.payload.settings);

    assert(spy.calledOnceWithExactly('123', '456'));
  });

  it('Should trigger selected audio playlist when item selector is set to "null" ', async () => {
    const json = FakeStreamDeck.generateJSON(Constants.AUDIO_ACTION_ID, { audioPlaylistSelector: '123', audioItemSelector: 'null' });
    const spy = sinon.spy(apiManager, 'triggerAudio');

    await action.performAction(json.payload.settings);

    assert(spy.calledOnceWithExactly('123', null));
  });

  it('Should throw an error when performAction is called with no audioPlaylistSelector in settings', async () => {
    const json = FakeStreamDeck.generateJSON(Constants.AUDIO_ACTION_ID, {});

    try {
      await action.performAction(json.payload.settings);
      assert(false, 'Error should have been thrown');
    } catch (error) {
      assert(error.message === 'audioPlaylistSelector not defined in settings');
    }
  });

  it('Should get playlists when `pollForData` is called and should send flattened playlists to property inspector', async () => {
    const spyOnGetPlaylists = sinon.spy(apiManager, 'getAudioPlaylists');
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');

    await action.pollForData('context', 'action');

    assert(spyOnGetPlaylists.calledOnceWithExactly());
    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availablePlaylists.length === 2);
  });

  it('Should send null for `availablePlaylists` to property inspector when error is thrown', async () => {
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');
    apiManager.getAudioPlaylists = () => {
      throw new Error();
    };

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.AUDIO_ACTION_ID);

    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availablePlaylists === null);
  });

  it('Should fetch and return the audio items for the selected playlist when onSendToPlugin is called with audioPlaylistSelector', async () => {
    const json = {
      payload: {
        sdpi_collection: { key: 'audioPlaylistSelector', value: '123' },
      },
      context: '9BD1ADCBFAA285E26F491B263B5B5237',
      action: Constants.AUDIO_ACTION_ID,
    };
    const spy = sinon.spy(apiManager, 'getAudioItemsInPlaylist');
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');

    await action.onSendToPlugin(json);

    assert(spy.calledOnceWithExactly('123'));
    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].selectedPlaylistAudioItems.items.length === 2);
  });

  it('Should send null for `selectedPlaylistAudioItems` to property inspector when error is thrown', async () => {
    const json = {
      payload: {
        sdpi_collection: { key: 'audioPlaylistSelector', value: '123' },
      },
      context: '9BD1ADCBFAA285E26F491B263B5B5237',
      action: Constants.AUDIO_ACTION_ID,
    };
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');
    apiManager.getAudioItemsInPlaylist = () => {
      throw new Error();
    };

    await action.onSendToPlugin(json);

    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].selectedPlaylistAudioItems == null);
  });

  it('Should not call the API method when onSendToPlugin is called with the incorrect payload', async () => {
    const json = {
      payload: {
        sdpi_collection: { key: 'notAudioPlaylistSelector', value: '123' },
      },
      context: '9BD1ADCBFAA285E26F491B263B5B5237',
      action: Constants.AUDIO_ACTION_ID,
    };
    const spy = sinon.spy(apiManager, 'getAudioItemsInPlaylist');

    await action.onSendToPlugin(json);

    assert(spy.notCalled);
  });
}

module.exports = test;
