/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const CaptureAction = require('../../js/actions/CaptureAction');

function test() {
  // *** Setup ***

  let apiManager;
  let action;

  let startCaptureJSON;
  let stopCaptureJSON;
  let toggleCaptureJSON;

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new CaptureAction(apiManager);

    startCaptureJSON = FakeStreamDeck.generateJSON(Constants.CAPTURE_ACTION_ID, { modeSelector: 'StartCapture' });
    stopCaptureJSON = FakeStreamDeck.generateJSON(Constants.CAPTURE_ACTION_ID, { modeSelector: 'StopCapture' });
    toggleCaptureJSON = FakeStreamDeck.generateJSON(Constants.CAPTURE_ACTION_ID, { modeSelector: 'ToggleCapture' });

    done();
  });

  // *** Tests ***

  it('Should initialize a new CaptureAction instance', () => {
    assert(action != null);
  });

  it('Should set defaults in settings if required', () => {
    const settings = {};

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === true);
    assert(settings.modeSelector === 'StartCapture');
    assert(settings['icon-state'] === 'start');
  });

  it('Should not set defaults in settings if not required', () => {
    const settings = {
      'icon-background-color': 'red',
      'icon-tint': 'lightIcon',
      actionColor: 'blue',
      modeSelector: 'StopCapture',
      'icon-state': 'stop',
    };

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === false);
    assert(settings.modeSelector === 'StopCapture');
    assert(settings['icon-state'] === 'stop');
  });

  it('Should start capture when performing action in start mode', () => {
    const spyOnStartCapture = sinon.spy(apiManager, 'startCapture');

    action.performAction(startCaptureJSON.payload.settings);

    assert(spyOnStartCapture.called);
  });

  it('Should stop capture when performing action in stop mode', () => {
    const spyOnStopCapture = sinon.spy(apiManager, 'stopCapture');

    action.performAction(stopCaptureJSON.payload.settings);

    assert(spyOnStopCapture.called);
  });

  it('Should update capture status and get settings when receiving callback from api manager', () => {
    const spyOnGetSettings = sinon.spy(action, 'getSettings');
    const captureStatus = {
      status: 'active',
      capture_time: '0:00:15',
      status_text: 'Active',
    };

    apiManager.callbackWithData(captureStatus);

    assert(action.captureStatus === captureStatus);
    assert(spyOnGetSettings.called);
  });

  it('Should stop capture when performing action in toggle mode and capture status is active', () => {
    const captureStatus = {
      status: 'active',
      capture_time: '0:00:15',
      status_text: 'Active',
    };
    apiManager.callbackWithData(captureStatus);
    const spyOnStopCapture = sinon.spy(apiManager, 'stopCapture');

    action.performAction(toggleCaptureJSON.payload.settings);

    assert(spyOnStopCapture.called);
  });

  it('Should stop capture when performing action in toggle mode and capture status is caution', () => {
    const captureStatus = {
      status: 'caution',
      capture_time: '0:00:15',
      status_text: 'Caution',
    };
    apiManager.callbackWithData(captureStatus);
    const spyOnStopCapture = sinon.spy(apiManager, 'stopCapture');

    action.performAction(toggleCaptureJSON.payload.settings);

    assert(spyOnStopCapture.called);
  });

  it('Should start capture when performing action in toggle mode and capture status is inactive', () => {
    const captureStatus = {
      status: 'inactive',
      capture_time: '0:00:15',
      status_text: 'Inactive',
    };
    apiManager.callbackWithData(captureStatus);
    const spyOnStartCapture = sinon.spy(apiManager, 'startCapture');

    action.performAction(toggleCaptureJSON.payload.settings);

    assert(spyOnStartCapture.called);
  });

  it('Should start capture when performing action in toggle mode and capture status is error', () => {
    const captureStatus = {
      status: 'error',
      capture_time: '0:00:15',
      status_text: 'Error',
    };
    apiManager.callbackWithData(captureStatus);
    const spyOnStartCapture = sinon.spy(apiManager, 'startCapture');

    action.performAction(toggleCaptureJSON.payload.settings);

    assert(spyOnStartCapture.called);
  });

  it('Should start capture when performing action in toggle mode and capture status is inactive', () => {
    const captureStatus = {
      status: 'inactive',
      capture_time: '0:00:15',
      status_text: 'Inactive',
    };
    apiManager.callbackWithData(captureStatus);
    const spyOnStartCapture = sinon.spy(apiManager, 'startCapture');

    action.performAction(toggleCaptureJSON.payload.settings);

    assert(spyOnStartCapture.called);
  });

  it('Should show alert when trying to perform toggle action with no status', async () => {
    const spyOnShowAlert = sinon.spy($SD.api, 'showAlert');
    await action.onKeyUp(toggleCaptureJSON);
    assert(spyOnShowAlert.called);
  });

  it('Should set action color to null when receiving settings and status is inactive', () => {
    const captureStatus = {
      status: 'inactive',
      capture_time: '0:00:15',
      status_text: 'Inactive',
    };
    apiManager.callbackWithData(captureStatus);

    action.onDidReceiveSettings(toggleCaptureJSON);

    assert(toggleCaptureJSON.payload.settings.stateColor === null);
  });

  it('Should set action color to active color when receiving settings and status is active', () => {
    const captureStatus = {
      status: 'active',
      capture_time: '0:00:15',
      status_text: 'Active',
    };
    apiManager.callbackWithData(captureStatus);

    action.onDidReceiveSettings(toggleCaptureJSON);

    assert(toggleCaptureJSON.payload.settings.stateColor === CaptureAction.ACTIVE_COLOR);
  });

  it('Should set action color to caution color when receiving settings and status is caution', () => {
    const captureStatus = {
      status: 'caution',
      capture_time: '0:00:15',
      status_text: 'Caution',
    };
    apiManager.callbackWithData(captureStatus);

    action.onDidReceiveSettings(toggleCaptureJSON);

    assert(toggleCaptureJSON.payload.settings.stateColor === CaptureAction.CAUTION_COLOR);
  });

  it('Should set action color to error color when receiving settings and status is error', () => {
    const captureStatus = {
      status: 'error',
      capture_time: '0:00:15',
      status_text: 'Error',
    };
    apiManager.callbackWithData(captureStatus);

    action.onDidReceiveSettings(toggleCaptureJSON);

    assert(toggleCaptureJSON.payload.settings.stateColor === CaptureAction.ERROR_COLOR);
  });

  it('Should set the action color to null when not in toggle mode', () => {
    const captureStatus = {
      status: 'caution',
      capture_time: '0:00:15',
      status_text: 'Caution',
    };
    apiManager.callbackWithData(captureStatus);

    action.onDidReceiveSettings(startCaptureJSON);

    assert(startCaptureJSON.payload.settings.stateColor === null);
  });

  it('Should set icon state to start when receiving settings and status is inactive', () => {
    const captureStatus = {
      status: 'inactive',
      capture_time: '0:00:15',
      status_text: 'Inactive',
    };
    apiManager.callbackWithData(captureStatus);

    action.onDidReceiveSettings(toggleCaptureJSON);

    assert(toggleCaptureJSON.payload.settings['icon-state'] === 'start');
  });

  it('Should set icon state to start when receiving settings and status is error', () => {
    const captureStatus = {
      status: 'error',
      capture_time: '0:00:15',
      status_text: 'Error',
    };
    apiManager.callbackWithData(captureStatus);

    action.onDidReceiveSettings(toggleCaptureJSON);

    assert(toggleCaptureJSON.payload.settings['icon-state'] === 'start');
  });

  it('Should set icon state to stop when receiving settings and status is active', () => {
    const captureStatus = {
      status: 'active',
      capture_time: '0:00:15',
      status_text: 'Active',
    };
    apiManager.callbackWithData(captureStatus);

    action.onDidReceiveSettings(toggleCaptureJSON);

    assert(toggleCaptureJSON.payload.settings['icon-state'] === 'stop');
  });

  it('Should set icon state to stop when receiving settings and status is caution', () => {
    const captureStatus = {
      status: 'caution',
      capture_time: '0:00:15',
      status_text: 'Caution',
    };
    apiManager.callbackWithData(captureStatus);

    action.onDidReceiveSettings(toggleCaptureJSON);

    assert(toggleCaptureJSON.payload.settings['icon-state'] === 'stop');
  });

  it('Should set icon state to start when receiving settings in StartCapture mode', () => {
    action.onDidReceiveSettings(startCaptureJSON);

    assert(startCaptureJSON.payload.settings['icon-state'] === 'start');
  });

  it('Should set icon state to stop when receiving settings in StopCapture mode', () => {
    action.onDidReceiveSettings(stopCaptureJSON);

    assert(stopCaptureJSON.payload.settings['icon-state'] === 'stop');
  });

  it('Should set the title to the capture time formatted to one leading 0 when in toggle mode', () => {
    const captureStatus = {
      status: 'caution',
      capture_time: '0:00:15',
      status_text: 'Caution',
    };
    apiManager.callbackWithData(captureStatus);
    const spyOnSetTitle = sinon.spy($SD.api, 'setTitle');

    action.onDidReceiveSettings(toggleCaptureJSON);

    assert(spyOnSetTitle.calledWith(toggleCaptureJSON.context, '0:15'));
  });

  it('Should not set the title when there is no capture status when in toggle mode', () => {
    const spyOnSetTitle = sinon.spy($SD.api, 'setTitle');

    action.onDidReceiveSettings(toggleCaptureJSON);

    assert(spyOnSetTitle.notCalled);
  });

  it('Should set the title to empty when not in toggle mode', () => {
    const spyOnSetTitle = sinon.spy($SD.api, 'setTitle');

    action.onDidReceiveSettings(startCaptureJSON);

    assert(spyOnSetTitle.calledWith(startCaptureJSON.context, ''));
  });
}

module.exports = test;
