/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const ClearAction = require('../../js/actions/ClearAction');

function test() {
  // *** Setup ***

  let apiManager;
  let action;

  let audioLayerJSON;
  let messageLayerJSON;
  let propLayerJSON;
  let annoucementLayerJSON;
  let slideLayerJSON;
  let mediaLayerJSON;
  let videoInputLayerJSON;
  let groupJSON;

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new ClearAction(apiManager);

    audioLayerJSON = FakeStreamDeck.generateJSON(Constants.CLEAR_ACTION_ID, { modeSelector: 'Audio-Layer' });
    messageLayerJSON = FakeStreamDeck.generateJSON(Constants.CLEAR_ACTION_ID, { modeSelector: 'Message-Layer' });
    propLayerJSON = FakeStreamDeck.generateJSON(Constants.CLEAR_ACTION_ID, { modeSelector: 'Prop-Layer' });
    annoucementLayerJSON = FakeStreamDeck.generateJSON(Constants.CLEAR_ACTION_ID, { modeSelector: 'Announcement-Layer' });
    slideLayerJSON = FakeStreamDeck.generateJSON(Constants.CLEAR_ACTION_ID, { modeSelector: 'Slide-Layer' });
    mediaLayerJSON = FakeStreamDeck.generateJSON(Constants.CLEAR_ACTION_ID, { modeSelector: 'Media-Layer' });
    videoInputLayerJSON = FakeStreamDeck.generateJSON(Constants.CLEAR_ACTION_ID, { modeSelector: 'Video-Input-Layer' });
    groupJSON = FakeStreamDeck.generateJSON(Constants.CLEAR_ACTION_ID, {
      modeSelector: '5C2E860C-166A-49E4-BFD4-C02E4BE447D4',
      selectedClearGroup: {
        id: { uuid: '5C2E860C-166A-49E4-BFD4-C02E4BE447D4', name: 'Clear All', index: 0 },
        icon: 'X',
        tint: {
          red: 0.7543836832046509, green: 0.049948517233133316, blue: 0.1292288452386856, alpha: 1,
        },
        layers: ['audio', 'audio-effects', 'message', 'prop', 'announcement', 'presentation', 'presentation_media', 'media', 'video_input'],
        stop_timeline_announcements: true,
        stop_timeline_presentation: true,
      },
    });

    done();
  });

  // *** Tests ***

  it('Should create a new ClearAction instance', () => {
    assert(action != null);
  });

  it('Should set defaults in settings if required', () => {
    const settings = {};

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === true);
    assert(settings['icon-state'] === 'X');
  });

  it('Should not set defaults in settings if not required', () => {
    const settings = {
      'icon-background-color': 'red',
      'icon-tint': 'lightIcon',
      actionColor: 'blue',
      modeSelector: 'StopCapture',
      'icon-state': 'eyeglasses',
    };

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === false);
    assert(settings['icon-state'] === 'eyeglasses');
  });

  it('Should get clear groups and send to PI when `pollForData` is called', async () => {
    const spyOnGet = sinon.spy(apiManager, 'getClearGroups');
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.CLEAR_ACTION_ID);

    assert(spyOnGet.called);
    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableClearGroups.length === 5);
  });

  it('Should send null to PI when `pollForData` is called and an error is thrown when getting data', async () => {
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');
    apiManager.getClearGroups = () => { throw new Error(); };

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.CLEAR_ACTION_ID);

    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableClearGroups === null);
  });

  // Triggering

  it('Should call triggerClearGroup with a clear group selected when performing action', async () => {
    const spyOnTriggerClearGroup = sinon.spy(apiManager, 'triggerClearGroup');

    await action.performAction(groupJSON.payload.settings);

    assert(spyOnTriggerClearGroup.calledWith('5C2E860C-166A-49E4-BFD4-C02E4BE447D4'));
  });

  it('Should call triggerClearLayer with audio layer with that layer selected when performing action', async () => {
    const spyOnTriggerClearLayer = sinon.spy(apiManager, 'triggerClearLayer');

    await action.performAction(audioLayerJSON.payload.settings);

    assert(spyOnTriggerClearLayer.calledWith('audio'));
  });

  it('Should call triggerClearLayer with messages layer with that layer selected when performing action', async () => {
    const spyOnTriggerClearLayer = sinon.spy(apiManager, 'triggerClearLayer');

    await action.performAction(messageLayerJSON.payload.settings);

    assert(spyOnTriggerClearLayer.calledWith('messages'));
  });

  it('Should call triggerClearLayer with props layer with that layer selected when performing action', async () => {
    const spyOnTriggerClearLayer = sinon.spy(apiManager, 'triggerClearLayer');

    await action.performAction(propLayerJSON.payload.settings);

    assert(spyOnTriggerClearLayer.calledWith('props'));
  });

  it('Should call triggerClearLayer with announcments layer with that layer selected when performing action', async () => {
    const spyOnTriggerClearLayer = sinon.spy(apiManager, 'triggerClearLayer');

    await action.performAction(annoucementLayerJSON.payload.settings);

    assert(spyOnTriggerClearLayer.calledWith('announcements'));
  });

  it('Should call triggerClearLayer with slide layer with that layer selected when performing action', async () => {
    const spyOnTriggerClearLayer = sinon.spy(apiManager, 'triggerClearLayer');

    await action.performAction(slideLayerJSON.payload.settings);

    assert(spyOnTriggerClearLayer.calledWith('slide'));
  });

  it('Should call triggerClearLayer with media layer with that layer selected when performing action', async () => {
    const spyOnTriggerClearLayer = sinon.spy(apiManager, 'triggerClearLayer');

    await action.performAction(mediaLayerJSON.payload.settings);

    assert(spyOnTriggerClearLayer.calledWith('media'));
  });

  it('Should call triggerClearLayer with announcments layer with that layer selected when performing action', async () => {
    const spyOnTriggerClearLayer = sinon.spy(apiManager, 'triggerClearLayer');

    await action.performAction(videoInputLayerJSON.payload.settings);

    assert(spyOnTriggerClearLayer.calledWith('video_input'));
  });

  // Status

  it('Should update layer status and get settings when receiving callback from api manager', () => {
    const spyOnGetSettings = sinon.spy(action, 'getSettings');
    const layerStatus = {
      video_input: false,
      media: false,
      slide: true,
      announcements: false,
      props: false,
      messages: false,
      audio: false,
    };

    apiManager.callbackWithData(layerStatus);

    assert(action.layerStatus === layerStatus);
    assert(spyOnGetSettings.called);
  });

  it('Should set action color to red when there is content to clear on the audio layer', () => {
    const layerStatus = {
      video_input: false,
      media: false,
      slide: false,
      announcements: false,
      props: false,
      messages: false,
      audio: true,
    };
    apiManager.callbackWithData(layerStatus);

    action.onDidReceiveSettings(audioLayerJSON);

    assert(audioLayerJSON.payload.settings.stateColor === ClearAction.ACTIVE_CONTENT_COLOR);
  });

  it('Should set action color to red when there is content to clear on the messages layer', () => {
    const layerStatus = {
      video_input: false,
      media: false,
      slide: false,
      announcements: false,
      props: false,
      messages: true,
      audio: false,
    };
    apiManager.callbackWithData(layerStatus);

    action.onDidReceiveSettings(messageLayerJSON);

    assert(messageLayerJSON.payload.settings.stateColor === ClearAction.ACTIVE_CONTENT_COLOR);
  });

  it('Should set action color to red when there is content to clear on the props layer', () => {
    const layerStatus = {
      video_input: false,
      media: false,
      slide: false,
      announcements: false,
      props: true,
      messages: false,
      audio: false,
    };
    apiManager.callbackWithData(layerStatus);

    action.onDidReceiveSettings(propLayerJSON);

    assert(propLayerJSON.payload.settings.stateColor === ClearAction.ACTIVE_CONTENT_COLOR);
  });

  it('Should set action color to red when there is content to clear on the announcements layer', () => {
    const layerStatus = {
      video_input: false,
      media: false,
      slide: false,
      announcements: true,
      props: false,
      messages: false,
      audio: false,
    };
    apiManager.callbackWithData(layerStatus);

    action.onDidReceiveSettings(annoucementLayerJSON);

    assert(annoucementLayerJSON.payload.settings.stateColor === ClearAction.ACTIVE_CONTENT_COLOR);
  });

  it('Should set action color to red when there is content to clear on the slide layer', () => {
    const layerStatus = {
      video_input: false,
      media: false,
      slide: true,
      announcements: false,
      props: false,
      messages: false,
      audio: false,
    };
    apiManager.callbackWithData(layerStatus);

    action.onDidReceiveSettings(slideLayerJSON);

    assert(slideLayerJSON.payload.settings.stateColor === ClearAction.ACTIVE_CONTENT_COLOR);
  });

  it('Should set action color to red when there is content to clear on the media layer', () => {
    const layerStatus = {
      video_input: false,
      media: true,
      slide: false,
      announcements: false,
      props: false,
      messages: false,
      audio: false,
    };
    apiManager.callbackWithData(layerStatus);

    action.onDidReceiveSettings(mediaLayerJSON);

    assert(mediaLayerJSON.payload.settings.stateColor === ClearAction.ACTIVE_CONTENT_COLOR);
  });

  it('Should set action color to red when there is content to clear on the video input layer', () => {
    const layerStatus = {
      video_input: true,
      media: false,
      slide: false,
      announcements: false,
      props: false,
      messages: false,
      audio: false,
    };
    apiManager.callbackWithData(layerStatus);

    action.onDidReceiveSettings(videoInputLayerJSON);

    assert(videoInputLayerJSON.payload.settings.stateColor === ClearAction.ACTIVE_CONTENT_COLOR);
  });

  it('Should set action color to red when there is content to clear on the slide layer', () => {
    const layerStatus = {
      video_input: false,
      media: false,
      slide: true,
      announcements: false,
      props: false,
      messages: false,
      audio: false,
    };
    apiManager.callbackWithData(layerStatus);

    action.onDidReceiveSettings(slideLayerJSON);

    assert(slideLayerJSON.payload.settings.stateColor === ClearAction.ACTIVE_CONTENT_COLOR);
  });

  it('Should set action color to null when there is not content to clear on the slide layer', () => {
    const layerStatus = {
      video_input: false,
      media: false,
      slide: false,
      announcements: false,
      props: false,
      messages: false,
      audio: false,
    };
    apiManager.callbackWithData(layerStatus);

    action.onDidReceiveSettings(slideLayerJSON);

    assert(slideLayerJSON.payload.settings.stateColor === null);
  });

  it('Should set action color to red when there is content to clear in the selected clear group', () => {
    const layerStatus = {
      video_input: false,
      media: false,
      slide: true,
      announcements: false,
      props: false,
      messages: false,
      audio: false,
    };
    apiManager.callbackWithData(layerStatus);

    action.onDidReceiveSettings(groupJSON);

    assert(groupJSON.payload.settings.stateColor === ClearAction.ACTIVE_CONTENT_COLOR);
  });

  it('Should set action color to null when there is not content to clear in the selected clear group', () => {
    const layerStatus = {
      video_input: false,
      media: false,
      slide: false,
      announcements: false,
      props: false,
      messages: false,
      audio: false,
    };
    apiManager.callbackWithData(layerStatus);

    action.onDidReceiveSettings(groupJSON);

    assert(groupJSON.payload.settings.stateColor === null);
  });
}

module.exports = test;
