/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const GroupAction = require('../../js/actions/GroupAction');

function test() {
  // *** Setup ***

  let apiManager;
  let action;

  let verseJSON;

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new GroupAction(apiManager);
    verseJSON = FakeStreamDeck.generateJSON(Constants.GROUP_ACTION_ID, {
      groupSelector: 'Verse 1',
      selectedGroup: {
        id: {
          uuid: 'A64F5355-9094-4797-B4BE-DA8C005D8DCC',
          name: 'Verse 1',
          index: 0,
        },
        color: {
          red: 0.0,
          green: 0.46666666865348816,
          blue: 0.800000011920929,
          alpha: 1.0,
        },
      },
    });

    done();
  });

  // *** Tests ***

  it('Should initialize a new Group Action instance', () => {
    assert(action != null);
  });

  it('Should call api manager with group to trigger when performing action', () => {
    const spyOnTriggerGroup = sinon.spy(apiManager, 'triggerGroup');

    action.performAction(verseJSON.payload.settings);

    assert(spyOnTriggerGroup.calledWith('Verse 1'));
  });

  it('Should throw an error if group selector is not defined when performing action', async () => {
    await assert.rejects(action.performAction({}), Error);
  });

  it('Should set title `onDidReceiveSettings`', () => {
    const spyOnSetTitle = sinon.spy($SD.api, 'setTitle');

    action.onDidReceiveSettings(verseJSON);

    assert(spyOnSetTitle.calledWith(verseJSON.context, 'Verse 1'));
  });

  it('Should set title `onWillAppear`', () => {
    const spyOnSetTitle = sinon.spy($SD.api, 'setTitle');

    action.onWillAppear(verseJSON);

    assert(spyOnSetTitle.calledWith(verseJSON.context, 'Verse 1'));
  });

  it('Should not set title `onWillAppear` if there is no selected group in settings', () => {
    const json = FakeStreamDeck.generateJSON(Constants.GROUP_ACTION_ID);
    const spyOnSetTitle = sinon.spy($SD.api, 'setTitle');

    action.onWillAppear(json);

    assert(spyOnSetTitle.notCalled);
  });

  it('Should set defaults in settings if required', () => {
    const settings = {};

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === true);
    assert(settings['icon-state'] === 'trigger');
  });

  it('Should not set defaults in settings if not required', () => {
    const settings = {
      'icon-background-color': 'red',
      'icon-tint': 'lightIcon',
      actionColor: 'blue',
      'icon-state': 'trigger',
    };

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === false);
  });

  it('Should get groups and send to PI when `pollForData` is called', async () => {
    const spyOnGet = sinon.spy(apiManager, 'getGroups');
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.GROUP_ACTION_ID);

    assert(spyOnGet.called);
    assert(spyOnSendToPI.args[0][1].availableGroups.length === 2);
  });

  it('Should send null to PI when `pollForData` is called and an error is thrown when getting data', async () => {
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');
    apiManager.getGroups = () => { throw new Error(); };

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.GROUP_ACTION_ID);

    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableGroups === null);
  });
}

module.exports = test;
