/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const PropAction = require('../../js/actions/PropAction');

function test() {
  // *** Setup ***

  let apiManager;
  let action;

  let triggerJSON;
  let clearJSON;
  let toggleJSON;

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new PropAction(apiManager);

    triggerJSON = FakeStreamDeck.generateJSON(Constants.PROP_ACTION_ID, {
      modeSelector: 'TriggerProp',
      propSelector: 'DAA83F50-B535-4A9E-826A-4DD2C4BA5082',
    });

    clearJSON = FakeStreamDeck.generateJSON(Constants.PROP_ACTION_ID, {
      modeSelector: 'ClearProp',
      propSelector: 'DAA83F50-B535-4A9E-826A-4DD2C4BA5082',
    });

    toggleJSON = FakeStreamDeck.generateJSON(Constants.PROP_ACTION_ID, {
      modeSelector: 'ToggleProp',
      propSelector: 'DAA83F50-B535-4A9E-826A-4DD2C4BA5082',
    });

    done();
  });

  // *** Tests ***

  it('Should create a new instance of PropAction', () => {
    assert(action != null);
  });

  it('Should set defaults in settings if required', () => {
    const settings = {};

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === true);
    assert(settings['icon-state'] === 'trigger');
    assert(settings.modeSelector === 'TriggerProp');
  });

  it('Should not set defaults in settings if not required', () => {
    const settings = {
      'icon-background-color': 'red',
      'icon-tint': 'lightIcon',
      actionColor: 'blue',
      'icon-state': 'trigger',
      modeSelector: 'TriggerProp',
    };

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === false);
  });

  it('Should get props and send to PI when `pollForData` is called', async () => {
    const spyOnGetProps = sinon.spy(apiManager, 'getProps');
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.PROP_ACTION_ID);

    assert(spyOnGetProps.called);
    assert(spyOnSendToPI.args[0][1].availableProps.length === 3);
  });

  it('Should send null to PI when `pollForData` is called and an error is thrown when getting data', async () => {
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');
    apiManager.getProps = () => { throw new Error(); };

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.PROP_ACTION_ID);

    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableProps === null);
  });

  it('Should `triggerProp` when performing action in trigger mode', async () => {
    const spyOnTriggerProp = sinon.spy(apiManager, 'triggerProp');

    await action.performAction(triggerJSON.payload.settings);

    assert(spyOnTriggerProp.calledWith('DAA83F50-B535-4A9E-826A-4DD2C4BA5082'));
  });

  it('Should `clearProp` when performing action in clear mode', async () => {
    const spyOnClearProp = sinon.spy(apiManager, 'clearProp');

    await action.performAction(clearJSON.payload.settings);

    assert(spyOnClearProp.calledWith('DAA83F50-B535-4A9E-826A-4DD2C4BA5082'));
  });

  it('Should `triggerProp` when performing action and prop is not active in toggle mode', async () => {
    const propStatus = [
      {
        id: {
          uuid: 'DAA83F50-B535-4A9E-826A-4DD2C4BA5082',
          name: 'Bravo Team',
          index: 0,
        },
        is_active: false,
      },
      {
        id: {
          uuid: 'CCB4C60D-6580-4E19-A3EB-512EA5D1E735',
          name: 'Something Else',
          index: 1,
        },
        is_active: false,
      },
      {
        id: {
          uuid: 'AB0BEFD0-63B6-42F2-A594-C85C78513F02',
          name: 'Prop',
          index: 2,
        },
        is_active: false,
      },
    ];
    apiManager.callbackWithData(propStatus);
    const spyOnTriggerProp = sinon.spy(apiManager, 'triggerProp');

    await action.performAction(toggleJSON.payload.settings);

    assert(spyOnTriggerProp.calledWith('DAA83F50-B535-4A9E-826A-4DD2C4BA5082'));
  });

  it('Should `clearProp` when performing action and prop is not active in toggle mode', async () => {
    const propStatus = [
      {
        id: {
          uuid: 'DAA83F50-B535-4A9E-826A-4DD2C4BA5082',
          name: 'Bravo Team',
          index: 0,
        },
        is_active: true,
      },
      {
        id: {
          uuid: 'CCB4C60D-6580-4E19-A3EB-512EA5D1E735',
          name: 'Something Else',
          index: 1,
        },
        is_active: false,
      },
      {
        id: {
          uuid: 'AB0BEFD0-63B6-42F2-A594-C85C78513F02',
          name: 'Prop',
          index: 2,
        },
        is_active: false,
      },
    ];
    apiManager.callbackWithData(propStatus);
    const spyOnClearProp = sinon.spy(apiManager, 'clearProp');

    await action.performAction(toggleJSON.payload.settings);

    assert(spyOnClearProp.calledWith('DAA83F50-B535-4A9E-826A-4DD2C4BA5082'));
  });

  it('Should throw an error when performing action and prop is found in status in toggle mode', async () => {
    const propStatus = [
      // Bravo Team prop removed
      {
        id: {
          uuid: 'CCB4C60D-6580-4E19-A3EB-512EA5D1E735',
          name: 'Something Else',
          index: 1,
        },
        is_active: false,
      },
      {
        id: {
          uuid: 'AB0BEFD0-63B6-42F2-A594-C85C78513F02',
          name: 'Prop',
          index: 2,
        },
        is_active: false,
      },
    ];
    apiManager.callbackWithData(propStatus);

    await assert.rejects(action.performAction(toggleJSON.payload.settings), Error);
  });

  it('Should throw an error if prop selector is not defined when performing action', async () => {
    await assert.rejects(action.performAction({ modeSelector: 'ToggleProp' }), Error);
  });

  it('Should throw an error if mode selector is not defined when performing action', async () => {
    await assert.rejects(action.performAction({ propSelector: 'DAA83F50-B535-4A9E-826A-4DD2C4BA5082' }), Error);
  });

  it('Should throw an error if mode selector is invalid when performing action', async () => {
    await assert.rejects(action.performAction({ propSelector: 'DAA83F50-B535-4A9E-826A-4DD2C4BA5082', modeSelector: 'invalid' }), Error);
  });

  it('Should update prop status and get settings when receiving callback from api manager', () => {
    const spyOnGetSettings = sinon.spy(action, 'getSettings');
    const propStatus = [
      {
        id: {
          uuid: 'DAA83F50-B535-4A9E-826A-4DD2C4BA5082',
          name: 'Bravo Team',
          index: 0,
        },
        is_active: false,
      },
      {
        id: {
          uuid: 'CCB4C60D-6580-4E19-A3EB-512EA5D1E735',
          name: 'Something Else',
          index: 1,
        },
        is_active: false,
      },
      {
        id: {
          uuid: 'AB0BEFD0-63B6-42F2-A594-C85C78513F02',
          name: 'Prop',
          index: 2,
        },
        is_active: false,
      },
    ];

    apiManager.callbackWithData(propStatus);

    assert(action.propStatus === propStatus);
    assert(spyOnGetSettings.called);
  });

  it('Should set the icon to `clear` when the prop is active in toggle mode `onDidReceiveSettings`', () => {
    const propStatus = [
      {
        id: {
          uuid: 'DAA83F50-B535-4A9E-826A-4DD2C4BA5082',
          name: 'Bravo Team',
          index: 0,
        },
        is_active: true,
      },
      {
        id: {
          uuid: 'CCB4C60D-6580-4E19-A3EB-512EA5D1E735',
          name: 'Something Else',
          index: 1,
        },
        is_active: false,
      },
      {
        id: {
          uuid: 'AB0BEFD0-63B6-42F2-A594-C85C78513F02',
          name: 'Prop',
          index: 2,
        },
        is_active: false,
      },
    ];
    apiManager.callbackWithData(propStatus);

    action.onDidReceiveSettings(toggleJSON);

    assert(toggleJSON.payload.settings['icon-state'] === 'clear');
  });

  it('Should set the icon to `trigger` when the prop is not active in toggle mode `onDidReceiveSettings`', () => {
    const propStatus = [
      {
        id: {
          uuid: 'DAA83F50-B535-4A9E-826A-4DD2C4BA5082',
          name: 'Bravo Team',
          index: 0,
        },
        is_active: false,
      },
      {
        id: {
          uuid: 'CCB4C60D-6580-4E19-A3EB-512EA5D1E735',
          name: 'Something Else',
          index: 1,
        },
        is_active: false,
      },
      {
        id: {
          uuid: 'AB0BEFD0-63B6-42F2-A594-C85C78513F02',
          name: 'Prop',
          index: 2,
        },
        is_active: false,
      },
    ];
    apiManager.callbackWithData(propStatus);

    action.onDidReceiveSettings(toggleJSON);

    assert(toggleJSON.payload.settings['icon-state'] === 'trigger');
  });

  it('Should not change the icon when the prop is not found in status in toggle mode `onDidReceiveSettings`', () => {
    const propStatus = [
      // Bravo Team prop removed
      {
        id: {
          uuid: 'CCB4C60D-6580-4E19-A3EB-512EA5D1E735',
          name: 'Something Else',
          index: 1,
        },
        is_active: false,
      },
      {
        id: {
          uuid: 'AB0BEFD0-63B6-42F2-A594-C85C78513F02',
          name: 'Prop',
          index: 2,
        },
        is_active: false,
      },
    ];
    apiManager.callbackWithData(propStatus);

    toggleJSON.payload.settings['icon-state'] = 'trigger';
    action.onDidReceiveSettings(toggleJSON);
    assert(toggleJSON.payload.settings['icon-state'] === 'trigger');

    toggleJSON.payload.settings['icon-state'] = 'clear';
    action.onDidReceiveSettings(toggleJSON);
    assert(toggleJSON.payload.settings['icon-state'] === 'clear');
  });

  it('Should set the icon to `trigger` in trigger mode `onDidReceiveSettings`', () => {
    action.onDidReceiveSettings(triggerJSON);

    assert(triggerJSON.payload.settings['icon-state'] === 'trigger');
  });

  it('Should set the icon to `clear` in clear mode `onDidReceiveSettings`', () => {
    action.onDidReceiveSettings(clearJSON);

    assert(clearJSON.payload.settings['icon-state'] === 'clear');
  });
}

module.exports = test;
