/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const StageAction = require('../../js/actions/StageAction');

function test() {
  // *** Setup ***

  let apiManager;
  let action;

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new StageAction(apiManager);
    done();
  });

  // *** Tests ***

  it('Should initialize a new StageAction instance', () => {
    assert(new StageAction(new FakeAPIManager()) != null);
  });

  it('Should set defaults in settings if required', () => {
    const settings = {};

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === true);
    assert(settings.modeSelector === 'SetLayouts');
    assert(settings['icon-state'] === 'show');
    assert(settings.message === '');
    assert.deepStrictEqual(settings.stageLayoutMap, []);
  });

  it('Should not set defaults in settings if not required', () => {
    const settings = {
      'icon-background-color': 'red',
      'icon-tint': 'lightIcon',
      actionColor: 'blue',
      modeSelector: 'HideStageMessage',
      'icon-state': 'hide',
      message: 'Hello World',
      stageLayoutMap: [],
    };

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === false);
    assert(settings.modeSelector === 'HideStageMessage');
    assert(settings['icon-state'] === 'hide');
    assert(settings.message === 'Hello World');
  });

  it('Should update live stage message and get settings when receiving callback from api manager', () => {
    const spyOnGetSettings = sinon.spy(action, 'getSettings');
    const message = 'Hello World!';

    apiManager.callbackWithData(message);

    assert(action.liveStageMessage === message);
    assert(spyOnGetSettings.called);
  });

  it('Should set the icon to `hide` when there is a message to clear in and receiving settings in `ToggleStageMessage` mode', () => {
    const message = 'Hello World!';
    apiManager.callbackWithData(message);
    const json = FakeStreamDeck.generateJSON(Constants.STAGE_ACTION_ID, { modeSelector: 'ToggleStageMessage' });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'hide');
  });

  it('Should set the icon to `show` when there is no message displayed and receiving settings in `ToggleStageMessage` mode', () => {
    const message = '';
    apiManager.callbackWithData(message);
    const json = FakeStreamDeck.generateJSON(Constants.STAGE_ACTION_ID, { modeSelector: 'ToggleStageMessage' });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'show');
  });

  it('Should set the icon to `show` when receiving settings in `ShowStageMessage` mode', () => {
    const json = FakeStreamDeck.generateJSON(Constants.STAGE_ACTION_ID, { modeSelector: 'ShowStageMessage' });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'show');
  });

  it('Should set the icon to `show` when receiving settings in `SetLayouts` mode', () => {
    const json = FakeStreamDeck.generateJSON(Constants.STAGE_ACTION_ID, { modeSelector: 'SetLayouts' });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'show');
  });

  it('Should set the icon to `hide` when receiving settings in `HideStageMessage` mode', () => {
    const json = FakeStreamDeck.generateJSON(Constants.STAGE_ACTION_ID, { modeSelector: 'HideStageMessage' });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'hide');
  });

  it('Should not set the icon when receiving settings and mode selector is invalid', () => {
    const json = FakeStreamDeck.generateJSON(Constants.STAGE_ACTION_ID, { modeSelector: 'invalid' });

    action.onDidReceiveSettings(json);

    assert(typeof json.payload.settings['icon-state'] === 'undefined');
  });

  // Polling

  it('Should get screens and layouts when `pollForData` is called and should send them to property inspector', async () => {
    const spyOnGetScreens = sinon.spy(apiManager, 'getStageScreens');
    const spyOnGetLayouts = sinon.spy(apiManager, 'getStageLayouts');
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');

    await action.pollForData('context', 'action');

    assert(spyOnGetScreens.calledOnceWithExactly());
    assert(spyOnGetLayouts.calledOnceWithExactly());
    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableStageScreens.length === 2);
    assert(spyOnSendToPI.args[0][1].availableStageLayouts.length === 2);
  });

  it('Should send null for `availableStageScreens` and `availableStageLayouts` to property inspector when error is thrown', async () => {
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');

    apiManager.getStageScreens = () => {
      throw new Error();
    };
    await action.pollForData('context', 'action');

    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableStageScreens === null);
    assert(spyOnSendToPI.args[0][1].availableStageLayouts === null);
  });

  // Peform action

  it('Should pass `showStageMessage` a configured message when performing action in `ShowStageMessage` mode', () => {
    const spyOnShowStageMessage = sinon.spy(apiManager, 'showStageMessage');

    action.performAction({
      modeSelector: 'ShowStageMessage',
      message: 'TEST MESSAGE',
    });

    assert(spyOnShowStageMessage.calledWith('TEST MESSAGE'));
  });

  it('Should call `hideStageMessage` when performing action in `HideStageMessage` mode', () => {
    const spyOnHideStageMessage = sinon.spy(apiManager, 'hideStageMessage');

    action.performAction({ modeSelector: 'HideStageMessage' });

    assert(spyOnHideStageMessage.called);
  });

  it('Should pass `showStageMessage` a configured message when performing action in `ToggleStageMessage` mode and there is no message displayed', () => {
    apiManager.callbackWithData('');
    const spyOnShowStageMessage = sinon.spy(apiManager, 'showStageMessage');

    action.performAction({
      modeSelector: 'ToggleStageMessage',
      message: 'TEST MESSAGE',
    });

    assert(spyOnShowStageMessage.calledWith('TEST MESSAGE'));
  });

  it('Should call `hideStageMessage` when performing action in `ToggleStageMessage` mode and there is a message displayed', () => {
    apiManager.callbackWithData('Hello World');
    const spyOnHideStageMessage = sinon.spy(apiManager, 'hideStageMessage');

    action.performAction({
      modeSelector: 'ToggleStageMessage',
      message: 'TEST MESSAGE',
    });

    assert(spyOnHideStageMessage.called);
  });

  it('Should call `setStageLayouts` when performing action in `SetLayouts` mode', () => {
    const spyOnSetLayouts = sinon.spy(apiManager, 'setStageLayouts');

    action.performAction({
      modeSelector: 'SetLayouts',
      stageLayoutMap: [],
    });

    assert(spyOnSetLayouts.called);
  });

  it('Should throw an error if stage layout map is not defined when setting layouts', () => {
    assert.rejects(action.performAction({ modeSelector: 'SetLayouts' }), Error);
  });

  it('Should throw an error if mode selector is not defined when performing action', async () => {
    await assert.rejects(action.performAction({ message: '' }), Error);
  });

  it('Should throw an error if mode selector is invalid when performing action', async () => {
    await assert.rejects(action.performAction({ message: '', modeSelector: 'invalid' }), Error);
  });

  it('Should throw an error if message is not defined when performing action', async () => {
    await assert.rejects(action.performAction({ modeSelector: 'ShowStageMessage' }), Error);
  });
}

module.exports = test;
