/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const TimelineAction = require('../../js/actions/TimelineAction');

function test() {
  // *** Setup ***

  let apiManager;
  let action;

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new TimelineAction(apiManager);
    done();
  });

  // *** Tests ***

  it('Should create a new TimelineAction instance', () => {
    assert(action != null);
  });

  it('Should throw an error if `modeSelector` is not defined when performing action', async () => {
    await assert.rejects(action.performAction({}), Error);
  });

  it('Should throw an error if `modeSelector` is invalid when performing action', async () => {
    await assert.rejects(action.performAction({ modeSelector: 'Merp' }), Error);
  });

  it('Should update `timelineStatus` object and get settings when receiving callback from api manager', () => {
    const spyOnGetSettings = sinon.spy(action, 'getSettings');
    const timelineStatus = {
      is_running: true,
      current_time: 6.3516305685043335,
    };

    apiManager.callbackWithData(timelineStatus);

    assert(action.timelineStatus === timelineStatus);
    assert(spyOnGetSettings.called);
  });

  it('Should set defaults in settings if required', () => {
    const settings = {};

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === true);
    assert(settings.modeSelector === 'Play');
    assert(settings['icon-state'] === 'play');
  });

  it('Should not set defaults in settings if not required', () => {
    const settings = {
      'icon-background-color': 'red',
      'icon-tint': 'lightIcon',
      actionColor: 'blue',
      modeSelector: 'PlayStopTimeline',
      'icon-state': 'play',
    };

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === false);
    assert(settings.modeSelector === 'PlayStopTimeline');
    assert(settings['icon-state'] === 'play');
  });

  it('Should play when the `performAction` event fires with the actions `modeSelector` set to "Play" ', async () => {
    const json = FakeStreamDeck.generateJSON(Constants.TIMELINE_ACTION_ID, { modeSelector: 'Play' });
    const spyOnTimelinePlay = sinon.spy(apiManager, 'timelinePlay');

    await action.performAction(json.payload.settings);

    assert(spyOnTimelinePlay.called);
  });

  it('Should pause when the `performAction` event fires with the actions `modeSelector` set to "Stop" ', async () => {
    const json = FakeStreamDeck.generateJSON(Constants.TIMELINE_ACTION_ID, { modeSelector: 'Stop' });
    const spyOnTimelinePause = sinon.spy(apiManager, 'timelinePause');

    await action.performAction(json.payload.settings);

    assert(spyOnTimelinePause.called);
  });

  it('Should play when the `performAction` event fires with the actions `modeSelector` set to "PlayStopTimeline" and the actions `timelineStatus.is_running` set to false', async () => {
    const timelineStatus = {
      is_running: false,
      current_time: 0.0,
    };
    apiManager.callbackWithData(timelineStatus);
    const json = FakeStreamDeck.generateJSON(Constants.TIMELINE_ACTION_ID, { modeSelector: 'PlayStopTimeline' });
    const spyOnTimelinePause = sinon.spy(apiManager, 'timelinePlay');

    await action.performAction(json.payload.settings);

    assert(spyOnTimelinePause.called);
  });

  it('Should pause when the `performAction` event fires with the actions `modeSelector` set to "PlayStopTimeline" and the actions `timelineStatus.is_running` set to true', async () => {
    const timelineStatus = {
      is_running: true,
      current_time: 6.3516305685043335,
    };
    apiManager.callbackWithData(timelineStatus);
    const json = FakeStreamDeck.generateJSON(Constants.TIMELINE_ACTION_ID, { modeSelector: 'PlayStopTimeline' });
    const spyOnTimelinePause = sinon.spy(apiManager, 'timelinePause');

    await action.performAction(json.payload.settings);

    assert(spyOnTimelinePause.called);
  });

  it('Should rewind when the `performAction` event fires with the actions `modeSelector` set to "GoToBeginning" ', async () => {
    const json = FakeStreamDeck.generateJSON(Constants.TIMELINE_ACTION_ID, { modeSelector: 'GoToBeginning' });
    const spyOnTimelineRewind = sinon.spy(apiManager, 'timelineRewind');

    await action.performAction(json.payload.settings);

    assert(spyOnTimelineRewind.called);
  });

  it('Should set `icon-state` to play when actions `modeSelector` is set to "Play" ', () => {
    const json = FakeStreamDeck.generateJSON(Constants.TIMELINE_ACTION_ID, { modeSelector: 'Play' });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'play');
  });

  it('Should set `icon-state` to stop when actions `modeSelector` is set to "Stop" ', () => {
    const json = FakeStreamDeck.generateJSON(Constants.TIMELINE_ACTION_ID, { modeSelector: 'Stop' });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'stop');
  });

  it('Should set `icon-state` to play when actions `modeSelector` is set to "PlayStopTimeline" and timeline is not running', () => {
    const timelineStatus = {
      is_running: false,
      current_time: 0.0,
    };
    apiManager.callbackWithData(timelineStatus);
    const json = FakeStreamDeck.generateJSON(Constants.TIMELINE_ACTION_ID, { modeSelector: 'PlayStopTimeline' });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'play');
  });

  it('Should set `icon-state` to stop when actions `modeSelector` is set to "PlayStopTimeline" and timeline is running', () => {
    const timelineStatus = {
      is_running: true,
      current_time: 6.3516305685043335,
    };
    apiManager.callbackWithData(timelineStatus);
    const json = FakeStreamDeck.generateJSON(Constants.TIMELINE_ACTION_ID, { modeSelector: 'PlayStopTimeline' });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'stop');
  });

  it('Should set `icon-state` to goToBeginning when `modeSelector` is set to "GoToBeginning" ', () => {
    const json = FakeStreamDeck.generateJSON(Constants.TIMELINE_ACTION_ID, { modeSelector: 'GoToBeginning' });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'goToBeginning');
  });
}

module.exports = test;
