/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const TimerAction = require('../../js/actions/TimerAction');

function test() {
  // *** Setup ***

  let apiManager;
  let action;

  let startTimerJSON;
  let stopTimerJSON;
  let toggleStartStopTimerJSON;
  let resetTimerJSON;
  let incrementTimerJSON;
  const countdownConfiguration = {
    id: {
      uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
    },
    allows_overrun: true,
    countdown: {
      duration: 300,
    },
  };

  const countdownToTimeConfiguration = {
    id: {
      uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
    },
    allows_overrun: true,
    count_down_to_time: {
      time_of_day: 300,
      period: 'am',
    },
  };

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new TimerAction(apiManager);

    startTimerJSON = FakeStreamDeck.generateJSON(Constants.TIMER_ACTION_ID, {
      modeSelector: 'StartTimer',
      timerSelector: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
      timerTypeSelector: 'CountdownTimer',
      countdownDuration: 300,
      allowsOverrun: true,
    });
    stopTimerJSON = FakeStreamDeck.generateJSON(Constants.TIMER_ACTION_ID, {
      modeSelector: 'StopTimer',
      timerSelector: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
      timerTypeSelector: 'CountdownToTime',
      countdownToTime: 300,
      countdownToTimeFormatSelector: 'am',
      allowsOverrun: true,
    });
    toggleStartStopTimerJSON = FakeStreamDeck.generateJSON(Constants.TIMER_ACTION_ID, {
      modeSelector: 'StartStopTimer',
      timerSelector: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
    });
    resetTimerJSON = FakeStreamDeck.generateJSON(Constants.TIMER_ACTION_ID, {
      modeSelector: 'ResetTimer',
      timerSelector: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
    });
    incrementTimerJSON = FakeStreamDeck.generateJSON(Constants.TIMER_ACTION_ID, {
      modeSelector: 'IncrementTimer',
      timerSelector: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
      incrementAmount: 60,
    });

    done();
  });

  // *** Tests ***

  it('Should initialize a new Timer Action instance', () => {
    assert(action != null);
  });

  it('Should set defaults in settings if required', () => {
    const settings = {};

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === true);
    assert(settings.modeSelector === 'StartTimer');
    assert(settings['icon-state'] === 'start');
    assert(settings.shouldSetConfiguration === false);
    assert(settings.timerTypeSelector === 'CountdownTimer');
    assert(settings.incrementAmount === 0);
    assert(settings.countdownDuration === 0);
    assert(settings.countdownToTime === 12 * 60 * 60);
    assert(settings.countdownToTimeFormatSelector === 'am');
    assert(settings.elapsedStartTime === 0);
    assert(settings.elapsedEndTime === null);
    assert(settings.allowsOverrun === false);
  });

  it('Should not set defaults in settings if not required', () => {
    const settings = {
      'icon-background-color': 'red',
      'icon-tint': 'lightIcon',
      actionColor: 'blue',
      modeSelector: 'StartTimer',
      'icon-state': 'start',
      shouldSetConfiguration: true,
      timerTypeSelector: 'CountdownTimer',
      incrementAmount: 0,
      countdownDuration: 20,
      countdownToTime: 0,
      countdownToTimeFormatSelector: 'pm',
      elapsedStartTime: 0,
      elapsedEndTime: 0,
      allowsOverrun: true,
    };

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === false);
    assert(settings.modeSelector === 'StartTimer');
    assert(settings['icon-state'] === 'start');
  });

  it('Should get timers when `pollForData` is called and should call `sendToPropertyInspector`', async () => {
    const spyOnGet = sinon.spy(apiManager, 'getTimers');
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.TIMER_ACTION_ID);

    assert(spyOnGet.called);
    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableTimers.length === 2);
  });

  it('Should send null for `availableTimers` to property inspector when error is thrown', async () => {
    const spyOnSendToPI = sinon.spy($SD.api, 'sendToPropertyInspector');
    apiManager.getTimers = () => {
      throw new Error();
    };

    await action.pollForData('9BD1ADCBFAA285E26F491B263B5B5237', Constants.TIMER_ACTION_ID);

    assert(spyOnSendToPI.called);
    assert(spyOnSendToPI.args[0][1].availableTimers === null);
  });

  it('Should fire start timer when the keyUp event runs', () => {
    const spyOnStartTimer = sinon.spy(apiManager, 'startTimer');

    action.onKeyUp(startTimerJSON);

    assert(spyOnStartTimer.called);
  });

  it('Should fire start timer when the keyUp event runs', () => {
    const spyOnStartTimer = sinon.spy(apiManager, 'startTimer');

    action.onKeyUp(startTimerJSON);

    assert(spyOnStartTimer.calledOnceWithExactly(
      startTimerJSON.payload.settings.timerSelector,
      null,
    ));
  });

  it('Should fire start timer with configuration when the keyUp event runs', () => {
    startTimerJSON.payload.settings.shouldSetConfiguration = true;
    const spyOnStartTimer = sinon.spy(apiManager, 'startTimer');

    action.onKeyUp(startTimerJSON);

    assert(spyOnStartTimer.calledOnceWithExactly(
      startTimerJSON.payload.settings.timerSelector,
      countdownConfiguration,
    ));
  });

  it('Should fire stop timer when the keyUp event runs', () => {
    const spyOnStopTimer = sinon.spy(apiManager, 'stopTimer');

    action.onKeyUp(stopTimerJSON);

    assert(spyOnStopTimer.called);
  });

  it('Should fire stop timer with configuration when the keyUp event runs', () => {
    stopTimerJSON.payload.settings.shouldSetConfiguration = true;
    const spyOnStopTimer = sinon.spy(apiManager, 'stopTimer');

    action.onKeyUp(stopTimerJSON);

    assert(spyOnStopTimer.calledOnceWithExactly(
      stopTimerJSON.payload.settings.timerSelector,
      countdownToTimeConfiguration,
    ));
  });

  it('Should fire reset timer when the keyUp event runs', () => {
    const spyOnResetTimer = sinon.spy(apiManager, 'resetTimer');

    action.onKeyUp(resetTimerJSON);

    assert(spyOnResetTimer.called);
  });

  it('Should fire increment timer when the keyUp event runs', () => {
    const spyOnIncrementTimer = sinon.spy(apiManager, 'incrementTimer');

    action.onKeyUp(incrementTimerJSON);

    assert(spyOnIncrementTimer.called);
  });

  it('Should throw an error if `timerSelector` is not defined when performing action', async () => {
    await assert.rejects(action.performAction({}), Error);
  });

  it('Should throw an error if `incrementTime` is not defined when performing increment', async () => {
    await assert.rejects(action.performAction({ modeSelector: 'IncrementTimer', timerSelector: '22222222-C4B2-44F7-A55D-4CC913BB8A5D' }), Error);
  });

  it('Should throw an error if `modeSelector` is not defined when performing action', async () => {
    await assert.rejects(action.performAction({ timerSelector: '22222222-C4B2-44F7-A55D-4CC913BB8A5D' }), Error);
  });

  it('Should throw an error if `modeSelector` is invalid when performing action', async () => {
    await assert.rejects(action.performAction({
      modeSelector: 'Merp',
      timerSelector: '22222222-C4B2-44F7-A55D-4CC913BB8A5D',
    }), Error);
  });

  it('Should update timer status and get settings when receiving callback from api manager', () => {
    const spyOnGetSettings = sinon.spy(action, 'getSettings');
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'stopped',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];

    apiManager.callbackWithData(timerStatus);

    assert(action.timerStatus === timerStatus);
    assert(spyOnGetSettings.called);
  });

  it('Should start timer when the `performAction` event fires with the actions `modeSelector` set to "StartStopTimer" and the actions `timerStatus.state` for the selected timer is set to "stopped" ', async () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'stopped',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);
    const spyOnStartTimer = sinon.spy(apiManager, 'startTimer');

    await action.performAction(toggleStartStopTimerJSON.payload.settings);

    assert(spyOnStartTimer.called);
  });

  it('Should stop timer when the `performAction` event fires with the actions `modeSelector` set to "StartStopTimer" and the actions `timerStatus.state` for the selected timer is set to "running" ', async () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'running',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);
    const spyOnStopTimer = sinon.spy(apiManager, 'stopTimer');

    await action.performAction(toggleStartStopTimerJSON.payload.settings);

    assert(spyOnStopTimer.called);
  });

  // 'overrun' was used in 7.9, so we will still support this value
  it('Should stop timer when the `performAction` event fires with the actions `modeSelector` set to "StartStopTimer" and the actions `timerStatus.state` for the selected timer is set to "overrun" ', async () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'overrun',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);
    const spyOnStopTimer = sinon.spy(apiManager, 'stopTimer');

    await action.performAction(toggleStartStopTimerJSON.payload.settings);

    assert(spyOnStopTimer.called);
  });

  it('Should stop timer when the `performAction` event fires with the actions `modeSelector` set to "StartStopTimer" and the actions `timerStatus.state` for the selected timer is set to "overrunning" ', async () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'overrunning',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);
    const spyOnStopTimer = sinon.spy(apiManager, 'stopTimer');

    await action.performAction(toggleStartStopTimerJSON.payload.settings);

    assert(spyOnStopTimer.called);
  });

  it('Should start timer when the `performAction` event fires with the actions `modeSelector` set to "StartStopTimer" and the actions `timerStatus.state` for the selected timer is set to "overran" ', async () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'overran',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);
    const spyOnStartTimer = sinon.spy(apiManager, 'startTimer');

    await action.performAction(toggleStartStopTimerJSON.payload.settings);

    assert(spyOnStartTimer.called);
  });

  it('Should stop timer when the `performAction` event fires with the actions `modeSelector` set to "StartStopTimer" and the actions `timerStatus.state` for the selected timer is set to "complete" ', async () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'complete',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);
    const spyOnResetTimer = sinon.spy(apiManager, 'resetTimer');

    await action.performAction(toggleStartStopTimerJSON.payload.settings);

    assert(spyOnResetTimer.called);
  });

  it('Should set `icon-state` to "stop" when getting settings and timer state is set to "running" in toggle mode', () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'running',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);

    action.onDidReceiveSettings(toggleStartStopTimerJSON);

    assert(toggleStartStopTimerJSON.payload.settings['icon-state'] === 'stop');
    assert(toggleStartStopTimerJSON.payload.settings.stateColor === TimerAction.RUNNING_COLOR);
  });

  // 'overrun' was used in 7.9, so we will still support this value
  it('Should set `icon-state` to "stop" when getting settings and timer state is set to "overrun" in toggle mode', () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'overrun',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);

    action.onDidReceiveSettings(toggleStartStopTimerJSON);

    assert(toggleStartStopTimerJSON.payload.settings['icon-state'] === 'stop');
    assert(toggleStartStopTimerJSON.payload.settings.stateColor === TimerAction.OVERRUN_COLOR);
  });

  it('Should set `icon-state` to "stop" when getting settings and timer state is set to "overrunning" in toggle mode', () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'overrunning',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);

    action.onDidReceiveSettings(toggleStartStopTimerJSON);

    assert(toggleStartStopTimerJSON.payload.settings['icon-state'] === 'stop');
    assert(toggleStartStopTimerJSON.payload.settings.stateColor === TimerAction.OVERRUN_COLOR);
  });

  it('Should set `icon-state` to "start" when getting settings and timer state is set to "overran" in toggle mode', () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'overran',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);

    action.onDidReceiveSettings(toggleStartStopTimerJSON);

    assert(toggleStartStopTimerJSON.payload.settings['icon-state'] === 'start');
    assert(toggleStartStopTimerJSON.payload.settings.stateColor === TimerAction.OVERRUN_COLOR);
  });

  it('Should set `icon-state` to "start" when getting settings and timer state is set to "stopped" in toggle mode', () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'stopped',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);

    action.onDidReceiveSettings(toggleStartStopTimerJSON);

    assert(toggleStartStopTimerJSON.payload.settings['icon-state'] === 'start');
    assert(toggleStartStopTimerJSON.payload.settings.stateColor === null);
  });

  it('Should set `icon-state` to "reset" when getting settings and timer state is set to "complete" in toggle mode', () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'complete',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);

    action.onDidReceiveSettings(toggleStartStopTimerJSON);

    assert(toggleStartStopTimerJSON.payload.settings['icon-state'] === 'reset');
    assert(toggleStartStopTimerJSON.payload.settings.stateColor === TimerAction.EXPIRED_COLOR);
  });

  it('Should set icon state to stop when getting settings in stop mode', () => {
    action.onDidReceiveSettings(stopTimerJSON);

    assert(stopTimerJSON.payload.settings['icon-state'] === 'stop');
  });

  it('Should set icon state to start when getting settings in start mode', () => {
    action.onDidReceiveSettings(startTimerJSON);

    assert(startTimerJSON.payload.settings['icon-state'] === 'start');
  });

  it('Should set icon state to reset when getting settings in reset mode', () => {
    action.onDidReceiveSettings(resetTimerJSON);

    assert(resetTimerJSON.payload.settings['icon-state'] === 'reset');
  });

  it('Should set the title to the current timer time without unecessary leading zeros or trailing ms when in toggle mode', () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '00:05:00.00',
        state: 'stopped',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);
    const spyOnSetTitle = sinon.spy($SD.api, 'setTitle');

    action.onDidReceiveSettings(toggleStartStopTimerJSON);

    assert(spyOnSetTitle.calledWith(toggleStartStopTimerJSON.context, '05:00'));
  });

  it('Should set the title to the current negative timer time without unecessary leading zeros or trailing ms when in toggle mode', () => {
    const timerStatus = [
      {
        id: {
          uuid: '80E2CDE8-2435-4899-9E88-B884900B9D6B',
          name: '5 Min Countdown',
          index: 0,
        },
        time: '-00:00:23.00',
        state: 'stopped',
      },
      {
        id: {
          uuid: '0E782588-A883-47DD-948E-38D4F0D27BCC',
          name: 'Timer',
          index: 1,
        },
        time: '12:05:00.00',
        state: 'stopped',
      },
    ];
    apiManager.callbackWithData(timerStatus);
    const spyOnSetTitle = sinon.spy($SD.api, 'setTitle');

    action.onDidReceiveSettings(toggleStartStopTimerJSON);

    assert(spyOnSetTitle.calledWith(toggleStartStopTimerJSON.context, '-00:23'));
  });

  it('Should set the title to empty when not in toggle mode', () => {
    const spyOnSetTitle = sinon.spy($SD.api, 'setTitle');

    action.onDidReceiveSettings(startTimerJSON);

    assert(spyOnSetTitle.calledWith(startTimerJSON.context, ''));
  });
}

module.exports = test;
