/* eslint-disable import/no-extraneous-dependencies, no-console */
// eslint-disable-next-line no-unused-vars
/* global $SD, describe, it, beforeEach, afterEach */
const assert = require('assert');
const sinon = require('sinon');
const Constants = require('../../js/constants');
const FakeAPIManager = require('../fakes/FakeAPIManager');
const FakeStreamDeck = require('../fakes/FakeStreamDeck');
const TransportAction = require('../../js/actions/TransportAction');

function test() {
  // *** Setup ***

  let apiManager;
  let action;

  // eslint-disable-next-line prefer-arrow-callback, func-names
  beforeEach(function (done) {
    apiManager = new FakeAPIManager();
    action = new TransportAction(apiManager);
    done();
  });

  // *** Tests ***

  it('Should initialize a new Transport Action instance', () => {
    assert(new TransportAction(new FakeAPIManager()) != null);
  });

  it('Should set defaults in settings if required', () => {
    const settings = {};

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === true);
    assert(settings['icon-state'] === 'play');
    assert(settings.modeSelector === 'Play');
    assert(settings.transportLayerSelector === 'Presentation');
    assert(settings.time === 15);
  });

  it('Should not set defaults in settings if not required', () => {
    const settings = {
      'icon-background-color': 'red',
      'icon-tint': 'lightIcon',
      actionColor: 'blue',
      'icon-state': 'pause',
      modeSelector: 'Pause',
      transportLayerSelector: 'Announcements',
      time: 7,
    };

    const didSetDefaults = action.setDefaultsInSettings(settings);

    assert(didSetDefaults === false);
    assert(settings['icon-state'] === 'pause');
    assert(settings.modeSelector === 'Pause');
    assert(settings.transportLayerSelector === 'Announcements');
    assert(settings.time === 7);
  });

  it('Should update announcment layer status and get settings when receiving callback from api manager', () => {
    const spyOnGetSettings = sinon.spy(action, 'getSettings');
    const announcementStatus = {
      is_playing: true,
      uuid: {
        string: 'DA0B7017-E853-4005-936E-1CF09C3BE2E1',
      },
      name: 'Screen Recording 2022-02-17 at 7.40.09 AM',
      artist: '',
      audio_only: false,
      duration: 7.410067081451416,
    };

    apiManager.callbackWithData(announcementStatus);

    assert(action.layerStatus.presentation === announcementStatus);
    assert(spyOnGetSettings.called);
  });

  it('Should update audio layer status and get settings when receiving callback from api manager', () => {
    const spyOnGetSettings = sinon.spy(action, 'getSettings');
    const audioStatus = {
      is_playing: true,
      uuid: {
        string: 'DA0B7017-E853-4005-936E-1CF09C3BE2E1',
      },
      name: 'Screen Recording 2022-02-17 at 7.40.09 AM',
      artist: '',
      audio_only: false,
      duration: 7.410067081451416,
    };

    apiManager.callbackWithData(audioStatus);

    assert(action.layerStatus.presentation === audioStatus);
    assert(spyOnGetSettings.called);
  });

  it('Should update presentation layer status and get settings when receiving callback from api manager', () => {
    const spyOnGetSettings = sinon.spy(action, 'getSettings');
    const presentationStatus = {
      is_playing: true,
      uuid: {
        string: 'DA0B7017-E853-4005-936E-1CF09C3BE2E1',
      },
      name: 'Screen Recording 2022-02-17 at 7.40.09 AM',
      artist: '',
      audio_only: false,
      duration: 7.410067081451416,
    };

    apiManager.callbackWithData(presentationStatus);

    assert(action.layerStatus.presentation === presentationStatus);
    assert(spyOnGetSettings.called);
  });

  it('Should set the icon to `pause` when in `PlayPause` mode the presentation layer is playing', () => {
    const presentationStatus = {
      is_playing: true,
      uuid: {
        string: 'DA0B7017-E853-4005-936E-1CF09C3BE2E1',
      },
      name: 'Screen Recording 2022-02-17 at 7.40.09 AM',
      artist: '',
      audio_only: false,
      duration: 7.410067081451416,
    };
    apiManager.callbackWithData(presentationStatus);
    const json = FakeStreamDeck.generateJSON(Constants.TRANSPORT_ACTION_ID, {
      modeSelector: 'PlayPause',
      transportLayerSelector: 'Presentation',
    });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'pause');
  });

  it('Should set the icon to `play` when in `PlayPause` mode and the presentation layer is not playing', () => {
    const presentationStatus = {
      is_playing: false,
      uuid: {
        string: 'DA0B7017-E853-4005-936E-1CF09C3BE2E1',
      },
      name: 'Screen Recording 2022-02-17 at 7.40.09 AM',
      artist: '',
      audio_only: false,
      duration: 7.410067081451416,
    };
    apiManager.callbackWithData(presentationStatus);
    const json = FakeStreamDeck.generateJSON(Constants.TRANSPORT_ACTION_ID, {
      modeSelector: 'PlayPause',
      transportLayerSelector: 'Presentation',
    });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'play');
  });

  it('Should set the icon to `play` when in `PlayPause` mode and no status is available', () => {
    const json = FakeStreamDeck.generateJSON(Constants.TRANSPORT_ACTION_ID, {
      modeSelector: 'PlayPause',
      transportLayerSelector: 'Presentation',
      'icon-state': 'goToEnd',
    });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'play');
  });

  it('Should set the icon to `play` when in `Play` mode', () => {
    const json = FakeStreamDeck.generateJSON(Constants.TRANSPORT_ACTION_ID, {
      modeSelector: 'Play',
      transportLayerSelector: 'Presentation',
    });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'play');
  });

  it('Should set the icon to `pause` when in `Pause` mode', () => {
    const json = FakeStreamDeck.generateJSON(Constants.TRANSPORT_ACTION_ID, {
      modeSelector: 'Pause',
      transportLayerSelector: 'Presentation',
    });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'pause');
  });

  it('Should set the icon to `skipForward` when in `SkipForward` mode', () => {
    const json = FakeStreamDeck.generateJSON(Constants.TRANSPORT_ACTION_ID, {
      modeSelector: 'SkipForward',
      transportLayerSelector: 'Presentation',
    });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'skipForward');
  });

  it('Should set the icon to `skipBackward` when in `SkipBackward` mode', () => {
    const json = FakeStreamDeck.generateJSON(Constants.TRANSPORT_ACTION_ID, {
      modeSelector: 'SkipBackward',
      transportLayerSelector: 'Presentation',
    });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'skipBackward');
  });

  it('Should set the icon to `goToEnd` when in `GoToEnd` mode', () => {
    const json = FakeStreamDeck.generateJSON(Constants.TRANSPORT_ACTION_ID, {
      modeSelector: 'GoToEnd',
      transportLayerSelector: 'Presentation',
    });

    action.onDidReceiveSettings(json);

    assert(json.payload.settings['icon-state'] === 'goToEnd');
  });

  it('Should set title `onDidReceiveSettings` when mode is changed to `GoToEnd`', () => {
    const json = FakeStreamDeck.generateJSON(Constants.TRANSPORT_ACTION_ID, {
      modeSelector: 'GoToEnd',
      transportLayerSelector: 'Presentation',
      'icon-state': 'play',
      time: 15,
    });

    const spyOnSetTitle = sinon.spy($SD.api, 'setTitle');

    action.onDidReceiveSettings(json);

    assert(spyOnSetTitle.calledWith(json.context, 15));
  });

  it('Should clear title `onDidReceiveSettings` when mode is changed to `Play`', () => {
    const json = FakeStreamDeck.generateJSON(Constants.TRANSPORT_ACTION_ID, {
      modeSelector: 'Play',
      transportLayerSelector: 'Presentation',
      'icon-state': 'goToEnd',
      time: 15,
    });

    const spyOnSetTitle = sinon.spy($SD.api, 'setTitle');

    action.onDidReceiveSettings(json);

    assert(spyOnSetTitle.calledWith(json.context, ''));
  });

  it('Should set title `onWillAppear` when icon is `goToEnd`', () => {
    const json = FakeStreamDeck.generateJSON(Constants.TRANSPORT_ACTION_ID, {
      modeSelector: 'GoToEnd',
      transportLayerSelector: 'Presentation',
      'icon-state': 'goToEnd',
      time: 15,
    });

    const spyOnSetTitle = sinon.spy($SD.api, 'setTitle');

    action.onWillAppear(json);

    assert(spyOnSetTitle.calledWith(json.context, 15));
  });

  it('Should clear title `onWillAppear` when icon is `play`', () => {
    const json = FakeStreamDeck.generateJSON(Constants.TRANSPORT_ACTION_ID, {
      modeSelector: 'Play',
      transportLayerSelector: 'Presentation',
      'icon-state': 'play',
      time: 15,
    });

    const spyOnSetTitle = sinon.spy($SD.api, 'setTitle');

    action.onWillAppear(json);

    assert(spyOnSetTitle.calledWith(json.context, ''));
  });

  // Perform Action

  it('Should play on the presentation layer when performing action', () => {
    const spyOnPlay = sinon.spy(apiManager, 'transportPlay');

    action.performAction({
      modeSelector: 'Play',
      transportLayerSelector: 'Presentation',
    });

    assert(spyOnPlay.calledWith('presentation'));
  });

  it('Should play on the announcements layer when performing action', () => {
    const spyOnPlay = sinon.spy(apiManager, 'transportPlay');

    action.performAction({
      modeSelector: 'Play',
      transportLayerSelector: 'Announcements',
    });

    assert(spyOnPlay.calledWith('announcement'));
  });

  it('Should play on the audio layer when performing action', () => {
    const spyOnPlay = sinon.spy(apiManager, 'transportPlay');

    action.performAction({
      modeSelector: 'Play',
      transportLayerSelector: 'Audio',
    });

    assert(spyOnPlay.calledWith('audio'));
  });

  it('Should pause on the presentation layer when performing action', () => {
    const spyOnPause = sinon.spy(apiManager, 'transportPause');

    action.performAction({
      modeSelector: 'Pause',
      transportLayerSelector: 'Presentation',
    });

    assert(spyOnPause.calledWith('presentation'));
  });

  it('Should skip forward on the presentation layer with a 15 second offset when performing action', () => {
    const spyOnSkipForward = sinon.spy(apiManager, 'transportSkipForward');

    action.performAction({
      modeSelector: 'SkipForward',
      transportLayerSelector: 'Presentation',
      time: 15,
    });

    assert(spyOnSkipForward.calledWith('presentation', 15));
  });

  it('Should skip backward on the presentation layer with a 15 second offset when performing action', () => {
    const spyOnSkipBackward = sinon.spy(apiManager, 'transportSkipBackward');

    action.performAction({
      modeSelector: 'SkipBackward',
      transportLayerSelector: 'Presentation',
      time: 15,
    });

    assert(spyOnSkipBackward.calledWith('presentation', 15));
  });

  it('Should go to end on the presentation layer with a 15 second offset when performing action', () => {
    const spyOnGoToEnd = sinon.spy(apiManager, 'transportGoToEnd');

    action.performAction({
      modeSelector: 'GoToEnd',
      transportLayerSelector: 'Presentation',
      time: 15,
    });

    assert(spyOnGoToEnd.calledWith('presentation', 15));
  });

  it('Should play when performing toggle action and the presentation layer is not playing', () => {
    const presentationStatus = {
      is_playing: false,
      uuid: {
        string: 'DA0B7017-E853-4005-936E-1CF09C3BE2E1',
      },
      name: 'Screen Recording 2022-02-17 at 7.40.09 AM',
      artist: '',
      audio_only: false,
      duration: 7.410067081451416,
    };
    apiManager.callbackWithData(presentationStatus);

    const spyOnPlay = sinon.spy(apiManager, 'transportPlay');

    action.performAction({
      modeSelector: 'PlayPause',
      transportLayerSelector: 'Presentation',
    });

    assert(spyOnPlay.calledWith('presentation'));
  });

  it('Should pause when performing toggle action and the presentation layer is playing', () => {
    const presentationStatus = {
      is_playing: true,
      uuid: {
        string: 'DA0B7017-E853-4005-936E-1CF09C3BE2E1',
      },
      name: 'Screen Recording 2022-02-17 at 7.40.09 AM',
      artist: '',
      audio_only: false,
      duration: 7.410067081451416,
    };
    apiManager.callbackWithData(presentationStatus);

    const spyOnPause = sinon.spy(apiManager, 'transportPause');

    action.performAction({
      modeSelector: 'PlayPause',
      transportLayerSelector: 'Presentation',
    });

    assert(spyOnPause.calledWith('presentation'));
  });

  it('Should throw an error if layer selector is not defined when performing action', async () => {
    await assert.rejects(action.performAction({ modeSelector: 'PlayPause' }), Error);
  });

  it('Should throw an error if layer selector is invalid when performing action', async () => {
    await assert.rejects(action.performAction({
      modeSelector: 'PlayPause',
      transportLayerSelector: 'invalid',
    }), Error);
  });

  it('Should throw an error if mode selector is not defined when performing action', async () => {
    await assert.rejects(action.performAction({ transportLayerSelector: 'Presentation' }), Error);
  });

  it('Should throw an error if mode selector is invalid when performing action', async () => {
    await assert.rejects(action.performAction({
      modeSelector: 'invalid',
      transportLayerSelector: 'Presentation',
    }), Error);
  });
}

module.exports = test;
